package inria.paperoles.gestures;

import java.awt.Color;

import inria.paperoles.ui.GesturesCanvas;
import inria.paperoles.ui.GlobalParameters;
import inria.paperoles.ui.GraphPaper;
import inria.paperoles.ui.MultiplePageViewer;
import inria.paperoles.ui.PaperTemplate;
import inria.paperoles.ui.ScoreViewer;
import inria.paperoles.ui.TabletTemplate;
import papertoolkit.events.EventHandler;
import papertoolkit.events.ExtendedPenEvent;
import papertoolkit.events.PenEvent;
import papertoolkit.events.handlers.GestureHandler;
import papertoolkit.pen.gesture.dollar.DollarRecognizer;
import papertoolkit.pen.gesture.dollar.DollarRecognizer.RecognitionResult;
import papertoolkit.pen.ink.InkStroke;
import papertoolkit.util.DebugUtils;

public class InkEventHandler extends EventHandler {

	MultiplePageViewer mviewer;
	
	/**
	 * Keeps the most recent event, so that when pen up happens, we can give it an event that does not have
	 * ZERO coordinates.
	 */
	private PenEvent lastEvent;

	/**
	 * Keep a single stroke around... throw away older strokes.
	 */
	private InkStroke mostRecentStroke;

	private boolean penDownHappened = false;

	/**
	 * @return the latest stroke
	 */
	public InkStroke getStroke() {
		return mostRecentStroke;
	}
	
/*	public InkEventHandler(ScoreViewer scoreViewer){
		super();
		viewer = scoreViewer;
	}
	*/

	int currentPage = -1;
	int pageCounter = 0;
	
	public InkEventHandler(MultiplePageViewer scoreViewer){
		super();
		mviewer = scoreViewer;
	}
	
	
	public void reset(){
		currentPage = -1;
		pageCounter = 0;
	}
	

	private void arrivedAtViewer(ScoreViewer viewer, PenEvent lastSample, InkStroke stroke){
		if(stroke.getSamples().size() >= 3){
/*			if(!(lastSample instanceof ExtendedPenEvent)){
			
				if(viewer.hasActiveSurface()) {
					viewer.acceptStroke(new SingleInkStroke(stroke), false);
					viewer.recordStroke(new SingleInkStroke(stroke, 1/viewer.getScale(), 1/viewer.getScale(), 0, 0), false);					
				}
			}
			else{*/
			
			int pnum = ((ExtendedPenEvent)lastSample).getPageNum();
			if(pnum > currentPage){
				++pageCounter;
				currentPage = pnum;
			}
			if(pnum < currentPage){
				pageCounter = 1;
				currentPage = pnum;
			}
			
			if(viewer.matches(pageCounter))	{		
				PaperTemplate template;
				
				template = lastSample.getPenID().equals("tablet")?new TabletTemplate(viewer.isVertical()):viewer.getPaperTemplate();
				
				viewer.acceptStroke(new SingleInkStroke(stroke, viewer.getSheetSize(), viewer.isVertical(), template), true);
				viewer.recordStroke(new SingleInkStroke(stroke, viewer.getSheetSize(), viewer.isVertical(), template), true);
			}
	//		}
		}
	}
	
	public void strokeArrived(PenEvent lastSample, InkStroke stroke) {		
		
		for(ScoreViewer viewer:mviewer.getPageViewers()){
			arrivedAtViewer(viewer, lastSample, stroke);
		}
		
/*		for(ScoreViewer viewer:mviewer.getGraphViewers()){
			arrivedAtViewer(viewer, lastSample, stroke);
		}*/
	}


	public void handleEvent(PenEvent event) {
		if (event == null || event.getOriginalSample() == null) {
			DebugUtils.println("Event Object is NULL");
			return;
		}

		if (event.isTypePenDown()) {	
			mostRecentStroke = new InkStroke();
			penDownHappened = true;
		} else if (event.isTypePenUp()) {
			// really, this should always be true
			if (penDownHappened) {				
				if(event.getOriginalSample().getX() != lastEvent.getOriginalSample().getX()
						|| event.getOriginalSample().getY() != lastEvent.getOriginalSample().getY()){
					mostRecentStroke.addSample(event.getOriginalSample());
					lastEvent = event;
				}
				
				strokeArrived(lastEvent, mostRecentStroke);
				penDownHappened = false;
				return;
			}
		} else {
			if (mostRecentStroke == null) {
				// we must have missed the down... 
				DebugUtils.println("Most Recent Stroke was null...... assume this was a pendown");
				penDownHappened = true;
				mostRecentStroke = new InkStroke();
			}
		}
		
		mostRecentStroke.addSample(event.getOriginalSample());
		lastEvent = event;
	}


	public String toString() {
		return getClass() +"";
	}

}
