package inria.paperoles.gestures;

import inria.paperoles.rendering.StrokeRenderer;
import inria.paperoles.scoremodel.Chord;
import inria.paperoles.scoremodel.InteractionBox;
import inria.paperoles.scoremodel.ScorePart;
import inria.paperoles.scoremodel.Segment;
import inria.paperoles.scoremodel.VoiceLine;
import inria.paperoles.ui.GlobalParameters;
import inria.paperoles.ui.PaperTemplate;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.Collection;
import java.util.List;
import java.util.Vector;

import edu.umd.cs.piccolo.util.PBounds;

import papertoolkit.pen.PenSample;
import papertoolkit.pen.ink.InkStroke;
import papertoolkit.units.Units;

public class SingleInkStroke extends GeneralInkStroke{

	InkStroke ref;
	
	private static final long MAX_LAG = 1400;
	public SingleInkStroke(List<PenSample> samples) {
		super(samples);
		
		analyze();
	}
	
	public SingleInkStroke(List<PenSample> samples, double scale_x, double scale_y, double offsetX, double offsetY) {
		super(scaleSamples(samples, scale_x, scale_y, offsetX, offsetY));
		
		analyze();
	}
	
	public SingleInkStroke(Collection<PenSample> samples, Units referenceUnit) {
		super(samples, referenceUnit);
		
		analyze();
	}
	
	public SingleInkStroke(InkStroke stroke, double scale_x, double scale_y, double offsetX, double offsetY) {
		this(stroke.getSamples(), scale_x, scale_y, offsetX, offsetY);
		
		ref = stroke;
	}
	
	
	// change for a while....
	/*****
	 * 
	 * 
	 */
	public SingleInkStroke(InkStroke stroke, Dimension sheetSize, boolean isVertical,
			PaperTemplate ptemplate) {
		this(isVertical?stroke.getSamples():rotate90(stroke.getSamples(), 				
				sheetSize.width, sheetSize.height, true),  
				ptemplate.getXScale(), ptemplate.getYScale(), ptemplate.getInkOffsetX(), ptemplate.getInkOffsetY());
		
		ref = stroke;
	}
	
	
	/*
	public SingleInkStroke(InkStroke stroke, Dimension sheetSize, double scale) {
		this(GlobalParameters.VERTICAL_ORIENTATON?stroke.getSamples():rotate90(stroke.getSamples(), 
				sheetSize.width, sheetSize.height, true),  scale/1.14, scale/1.14, 118, -12);
		
		ref = stroke;
	}*/
	
	/*
	public SingleInkStroke(InkStroke stroke, Dimension sheetSize) {
		this(GlobalParameters.VERTICAL_ORIENTATON?stroke.getSamples():rotate90(stroke.getSamples(), 
				sheetSize.width, sheetSize.height, true),  1.18, 1.18, 0, 0
				GlobalParameters.VERTICAL_ORIENTATON?0:-50, GlobalParameters.VERTICAL_ORIENTATON?-50:0);
	}*/
	
	public SingleInkStroke(GeneralInkStroke stroke, int lastIndex) {
		super(stroke.getSamples().subList(0, lastIndex));
		
		StrokeRenderer renderer = new StrokeRenderer(this);
		setShape(renderer.getShape());
		
		ref = stroke;
	}
	
	public SingleInkStroke(InkStroke stroke) {
		this(stroke.getSamples(), 1, 1, 0, 0);
		//super(rotate90(stroke.getSamples(), size.width, size.height, true));
	
		analyze();
		
		ref = stroke;
	}

	private static List<PenSample> rotate90(List<PenSample> originalSamples, int width, int height, boolean clockwise)
	{
		Vector<PenSample>samples = new Vector<PenSample>();
		for(PenSample sample:originalSamples){
			PenSample tsample;
			
			if(clockwise) tsample = new PenSample(width - sample.y, sample.x, sample.force, sample.timestamp); 
			else tsample = new PenSample(sample.y, height - sample.x, sample.force, sample.timestamp); 
				
			samples.add(tsample);
		}
	
		return samples;
	}
	
	private static List<PenSample> scaleSamples(List<PenSample> originalSamples, double scale_x, double scale_y, double offsetX, double offsetY){
		Vector<PenSample>samples = new Vector<PenSample>();
		for(PenSample sample:originalSamples){
			PenSample tsample = new PenSample(sample.x*scale_x + offsetX, sample.y*scale_y + offsetY,
					sample.force, sample.timestamp); 
			samples.add(tsample);
		}
	
		return samples;
	}
	
	public Vector<SingleInkStroke> getStrokes(){
		Vector<SingleInkStroke> strokes = new Vector<SingleInkStroke>();
		strokes.add(this);
		
		return strokes;
	}
	
	public Collection<Shape> getShapes() {
		Vector<Shape> shapes = new Vector<Shape>();
		shapes.add(getShape());
		
		return shapes;
	}

	class Peek {
		boolean maximum;
		double x, y;
		PenSample sample;
		
		Peek(boolean maximum, double x, double y, int index){
			this.maximum = maximum;
			this.x = x;
			this.y = y;
			sample = getSamples().get(index);
		}
		
		public Point2D getPoint(){
			return new Point2D.Double(sample.getX(), sample.getY());
		}
		
		public boolean isMax(){
			return maximum;
		}
		
		public double getX(){
			return x;
		}
		
		public double getY(){
			return y;
		}		
		
		public String toString(){
			return (maximum?"max":"min") + ": " + x + ", "+ y;
		}
	}
	
	public Peek getFirstPeek(){
		return peeks.firstElement();
	}
	
	public Vector<Peek> getPeeks(){
		return peeks;
	}

	public double getEndsDistance(){
		return dist;
	}
	
	Shape shape;
	public void setShape(Shape shape){
		this.shape = shape;
	}
	
	public Shape getShape(){
		return shape;
	}
	
	Vector<Peek> peeks;
	double dist; // distance between starting and ending points
	
	
	public Rectangle2D getBounds(){ 
		return shape.getBounds2D();
	}
	
	private void analyze(){
		peeks = findPeeks(getTransformedInk(getStart(), getEnd()), 
				Math.max(SAMPLE_X, dist/10));
	}
	
	
	private Peek getArrowHead(){
		if(peeks.size() != 1 && peeks.size() != 2) return null;
		
		Peek peek = peeks.get(0);
		if(peeks.size() == 2 && peeks.get(1).getPoint().distance(getStart().getX(), getStart().getY()) < 10 ) peek = peeks.get(1);	
		else if(peeks.size() != 1) return null;
		
		if(dist > 5*Math.abs(peeks.get(0).y)) return null;
		
		return peek;
		//return peeks.get(0).getPoint();
	}
	
	
	public ArrowStroke getArrow(GeneralInkStroke stroke){
	
		if(!(stroke instanceof SingleInkStroke)) return null;
		SingleInkStroke head, tail;
		
		if(getFirstTimestamp() - stroke.getLastTimestamp() > MAX_LAG) return null;
		if(getEndsDistance() < ((SingleInkStroke)stroke).getEndsDistance()){
			head = this;
			tail = (SingleInkStroke)stroke;
		}
		else{
			head = (SingleInkStroke)stroke;
			tail = this;
		}

		//Point2D headPoint = head.getArrowHead();
		Peek peek = head.getArrowHead();
		
		if(peek == null) return null;
			
		//Peek peek = head.getFirstPeek();
		
		if((peek.getPoint().distance(tail.getFirstPoint()) < Math.min(40, Math.abs(peek.getY()) + 10) 
				|| peek.getPoint().distance(tail.getLastPoint()) < Math.min(40, Math.abs(peek.getY()) + 10))
				&& tail.intersects(new Line2D.Double(head.getFirstPoint(), head.getLastPoint()))
			){
		
			ArrowStroke newStroke = new ArrowStroke(tail, head);
			newStroke.setHeadPoint(peek.getPoint());
				
			return newStroke;
		}			
		
		return null;
	}

	
	Vector<Peek> findPeeks(Vector<Point2D> points, double sample_x){
		Vector<Peek> peeks = new Vector<Peek>();
		
		double y = points.get(0).getY();
		double x = points.get(0).getX();
		
		boolean increasing = true;
		Point2D p; 
		int i = 1;
		int index = 0;
		
		for(p  = points.get(1); i<points.size() - 1 && (Math.abs(p.getX() - x) < sample_x); 
			p  = points.get(++i));	
		
		increasing = (p.getY() >= y);
		x = p.getX();
		y = p.getY();
		index = i;
	
		for(i=i+1; i<points.size(); ++i){
			p  = points.get(i);
			
			if(Math.abs(p.getX() - x) < sample_x) continue;
			
			if(increasing && p.getY() < y){
				peeks.add(new Peek(true, x, y, index));
				increasing = false;
			}
			else if(!increasing && p.getY() > y){
				peeks.add(new Peek(false, x, y, index));
				increasing = true;
			}
				
			x = p.getX();
			y = p.getY();
			index = i;
		}
		
		return peeks;
	}
	
	
	Vector<Point2D> getTransformedInk(PenSample s1, PenSample s2){
		dist = Math.sqrt(Math.pow(s1.x - s2.x, 2) + Math.pow(s1.y - s2.y, 2));
		double cos = (s2.x - s1.x)/dist;
		double sin = (s1.y - s2.y)/dist;
		
		List<PenSample>  samples = getSamples();
		Vector<Point2D> tsamples = new Vector<Point2D>();

		for(int i=0; i<samples.size(); ++i){
			PenSample sample = samples.get(i);
			
			double x = (sample.x - s1.x)*cos - (sample.y - s1.y)*sin;
			double y = -(sample.x - s1.x)*sin - (sample.y - s1.y)*cos;
			
			Point2D point = new Point2D.Double(x, y);
			tsamples.add(point);
		}
		
		return tsamples;
	}

	public boolean intersects(PBounds bounds) {
		return shape.intersects(bounds);
	}

	SingleStrokeAnalysisResult result;
	public void setAnalysisResult(SingleStrokeAnalysisResult result) {
		this.result = result;
		
	}
	
	public SingleStrokeAnalysisResult getAnalysisResult() {
		return result;	
	}

	@Override
	public GeneralInkStroke getMirroredStroke(int axis) {
		Vector<PenSample> tsamples = new Vector<PenSample>();
		
		for(PenSample sample:getSamples()){
			PenSample tsample;
			
			if(axis == X_AXIS)
				tsample = new PenSample(sample.x, -sample.y, sample.force, sample.timestamp); 
			else 
				tsample = new PenSample(-sample.x, sample.y, sample.force, sample.timestamp);
				
			tsamples.add(tsample);
		}
		
		return new SingleInkStroke(tsamples);
	}

	// for the moment I don't try to change the resolution of the samples as the size of the stroke changes
	public GeneralInkStroke getScaledStroke(double scalex, double scaley) {
		return new SingleInkStroke(getSamples(), scalex, scaley, 0, 0);
	}

	@Override
	public GeneralInkStroke getRotatedStroke(double radians) {
		AffineTransform rotateTrans = new AffineTransform();
		rotateTrans.rotate(radians);
		
		Vector<PenSample> tsamples = new Vector<PenSample>();
		for(PenSample sample:getSamples()){
			Point2D p = new Point2D.Double(sample.getX(), sample.getY());
			Point2D pt = new Point2D.Double();
			rotateTrans.transform(p, pt);
			
			PenSample tsample = new PenSample(pt.getX(), pt.getY(), 
					sample.force, sample.timestamp);
			
			tsamples.add(tsample);
		}
		
		return new SingleInkStroke(tsamples);
	}
	
	@Override
	public GeneralInkStroke getTimeInversedStroke() {
		
		Object[]samples = getSamples().toArray();
		Vector<PenSample> tsamples = new Vector<PenSample>();
		
		for(int i=0; i<samples.length; ++i){
			PenSample sample = new PenSample(((PenSample)samples[samples.length - i - 1]).getX(), ((PenSample)samples[samples.length - i - 1]).getY(), 
					((PenSample)samples[samples.length - i - 1]).getForce(), ((PenSample)samples[i]).getTimestamp());
			
			tsamples.add(sample);
		}
		
		// TODO Auto-generated method stub
		return new SingleInkStroke(tsamples);
	}

	@Override
	// need to refine this... Horizontal lines can be really small...
	public GeneralInkStroke getWithCutEnds(int num1 , int num2) {
		Vector<PenSample> tsamples = new Vector<PenSample>();
		
		int i = 0;
		for(PenSample sample:getSamples()){
			if(++i<=num1) continue;
				
			PenSample tsample = new PenSample(sample.x, sample.y, 1 + sample.force, sample.timestamp); 
			tsamples.add(tsample);
			
			if(i > getSamples().size() - num2) break;
		}
		
		return new SingleInkStroke(tsamples);
	}

	@Override
	public GeneralInkStroke getRandomStroke() {
		// TODO Auto-generated method stu
		Vector<PenSample> tsamples = new Vector<PenSample>();
		
		for(PenSample sample:getSamples()){
			PenSample tsample = new PenSample(sample.x*Math.random(), sample.y*Math.random(), (int)(Math.random()*10*sample.force), sample.timestamp); 
			tsamples.add(tsample);
		}
		
		return new SingleInkStroke(tsamples);
	}

	@Override
	public boolean intersects(Line2D line) {
		PenSample p1 = null;  //System.err.println("1 >>> " + shape.intersects(line.getBounds()));
		for(PenSample sample:getSamples()){ //if(p1!=null) System.err.println("i >>> " + line.intersectsLine(p1.x, p1.y, sample.x, sample.y));
			if(p1 != null && line.intersectsLine(p1.x, p1.y, sample.x, sample.y)) return true;
			p1 = sample;
		}
		
		return false;
	}
	
	public boolean startsAbout(GeneralInkStroke stroke){
		Rectangle2D bounds = stroke.getBounds();
		if(bounds.getWidth() <=5)
			bounds = new Rectangle2D.Double(bounds.getX() - 3, bounds.getY(), 
					6, bounds.getHeight());
		else if(bounds.getHeight() <=5)
			bounds = new Rectangle2D.Double(bounds.getX(), bounds.getY() - 3, 
					bounds.getWidth(), 6);
		
		return (bounds.intersects(
				new Rectangle2D.Double(getStart().x-4, getStart().y-4, 8, 8)));
	}
	
	public boolean startsFrom(GeneralInkStroke stroke){
		return stroke.getShape().contains(new Point2D.Double(getStart().x, getStart().y));	
	}
	
	public boolean endsAbout(GeneralInkStroke stroke){
		Rectangle2D bounds = stroke.getBounds();
		if(bounds.getWidth() <=5)
			bounds = new Rectangle2D.Double(bounds.getX() - 3, bounds.getY(), 
					6, bounds.getHeight());
		else if(bounds.getHeight() <=5)
			bounds = new Rectangle2D.Double(bounds.getX(), bounds.getY() - 3, 
					bounds.getWidth(), 6);
		
		return (bounds.intersects(
				new Rectangle2D.Double(getEnd().x-4, getEnd().y-4, 8, 8)));
		//return (stroke.getBounds().contains(getEnd().x, getEnd().y)); 

	}
	
	public boolean endsTo(GeneralInkStroke stroke){
		
		return stroke.getShape().contains(new Point2D.Double(getEnd().x, getEnd().y));
	/*
		Rectangle2D bounds = stroke.getBounds();
		if(bounds.getWidth() <=5)
			bounds = new Rectangle2D.Double(bounds.getX() - 3, bounds.getY(), 
					6, bounds.getHeight());
		else if(bounds.getHeight() <=5)
			bounds = new Rectangle2D.Double(bounds.getX(), bounds.getY() - 3, 
					bounds.getWidth(), 6);
		
		return (bounds.intersects(
				new Rectangle2D.Double(getEnd().x-4, getEnd().y-4, 8, 8)));
		//return (stroke.getBounds().contains(getEnd().x, getEnd().y)); 
		  
	 */
	}

	@Override
	public boolean inShape(Shape shape) {
		for(PenSample sample:getSamples()){
			if(shape.contains(sample.x, sample.y)) return true;
		}
		
		return false;
	}

	public void setSmoothShape(Shape smoothShape) {
		this.smoothShape = smoothShape;
		
	}

	@Override
	public Shape getSmoothShape() {

		return smoothShape;
	}

	@Override
	public int getPigTailIndex() {
		Line2D line1 = null, line2 = null;
		
		List<PenSample> samples = getSamples();
		
		for(int i = samples.size() - 1; i>2; --i){ 
			line1 = new Line2D.Double(samples.get(i).getX(), samples.get(i).getY(),
					samples.get(i - 1).getX(), samples.get(i - 1).getY());
			
			for(int j = i - 2; j>1; --j){
				line2 = new Line2D.Double(samples.get(j).getX(), samples.get(j).getY(),
						samples.get(j - 1).getX(), samples.get(j - 1).getY());
				
				if(line1.intersectsLine(line2)) return j;
			}
		
		}
		
		return -1;
	}
	

	public InkStroke getRef(){
		return ref;
	}

	public ScorePart getNote(VoiceLine vline){
		double x = getShape().getBounds().getCenterX();
		
		double minx = 10000000;
		
		Chord c = null;
		for(Segment segment:vline.getSegments()){
			for(Chord chord:segment.getChords()){
				double _x = Math.abs(chord.getCenter().getX() - x);
				if(_x <= minx){
					minx = _x;
					c = chord;
				}
			}
		}
		
		return c;
	}
	
	public ScorePart[] getNotes(VoiceLine vline){
		double x1 = getShape().getBounds().getMinX();//Math.min(getFirstPoint().getX(), getLastPoint().getX()); 
		double x2 = getShape().getBounds().getMaxX();//Math.max(getFirstPoint().getX(), getLastPoint().getX());
		
		double minx1 = 10000000, minx2 = 10000000;
		
		Chord chords[] = new Chord[2];
		//Chord c1 = null, c2 = null;
		for(Segment segment:vline.getSegments()){
			for(Chord chord:segment.getChords()){
				double _x = Math.abs(chord.getCenter().getX() - x1);
				if(_x <= minx1){
					minx1 = _x;
					chords[0] = chord;
				}
				
				_x = Math.abs(chord.getCenter().getX() - x2);
				if(_x <= minx2){
					minx2 = _x;
					chords[1] = chord;
				}
			}
		}
		
		if(chords[0] !=null && chords[1] !=null) return chords;
		else return null;
	}

	public boolean isInside(PBounds bounds) {
		for(PenSample sample:getSamples()){
			if(!bounds.contains(sample.getX(), sample.getY()))
				return false;
			
		}
		return true;
	}

	@Override
	public double distanceFrom(GeneralInkStroke generalInkStroke) {
		// TODO Auto-generated method stub
		double minDist = 5000;
	
		//better consider SQR distances
		for(PenSample sample1:getSamples()){
			for(PenSample sample2:generalInkStroke.getSamples()){
				double dist = sample1.getDistanceFrom(sample2);
				
				if(dist<minDist) minDist = dist;
			}	
		}
		
		return minDist;
	}

	
}
