package inria.paperoles.gestures;

import inria.paperoles.rendering.StrokeRenderer;
import inria.paperoles.scoping.ScopeConstraint;
import inria.paperoles.scoremodel.GraphVoiceLine;
import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.scoremodel.ScoreVerticalPointer;
import inria.paperoles.scoremodel.Segment;
import inria.paperoles.scoremodel.VoiceLine;
//import inria.paperoles.ui.Scope;

import java.awt.Shape;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.Vector;

import papertoolkit.pen.ink.InkStroke;

public class SingleStrokeAnalysisResult {
	
	final static short VERTICAL = 0;
	final static short HORIZONTAL = 1;
	public final static short CLOSED_SELECTION = 1;
	public final static short GENERAL_CURVE = 2;
	public final static short ARROW = 3;
	
	public final static short LEFT = 1;
	public final static short RIGHT = 2;
	public final static short UP = 3;
	public final static short DOWN = 4;
	
//	Scope scope;
	short strokeClass;
//	Shape shape;
	SingleInkStroke stroke;
//	StrokeRenderer renderer;
	
	double angle;
	short direction;
	
	ScopeConstraint constraint;
	
	public SingleStrokeAnalysisResult(SingleInkStroke stroke){
		this.stroke = stroke;
	}
	
	public SingleInkStroke getStroke(){
		return stroke;
	}
	
	public Point2D getStartOfStroke(){
		return new Point2D.Double(stroke.getStart().x, stroke.getStart().y);
	}
	
	public Point2D getEndOfStroke(){
		return new Point2D.Double(stroke.getEnd().x, stroke.getEnd().y);
	}

	
	public void setClass(short strokeClass){
		this.strokeClass = strokeClass;
	}

	
	public short getStrokeClass(){
		return strokeClass;
	}

	public void setAngle(double strokeAngle) {
		angle = strokeAngle;
	
		if(Math.abs(angle) <= 45) direction = RIGHT;
		else if(Math.abs(angle) >= 135) direction = LEFT;
		else if(angle > 45) direction = UP;
		else direction = DOWN;
			
	}
	
	public double getAngle(){
		return angle;
	}

	public short getDirection(){
		return direction;
	}
	
	
	public boolean isRight(){
		return direction == RIGHT;
	}
	
	public boolean isLeft(){
		return direction == LEFT;
	}
	
	public boolean isHorizontal(){ 
		Rectangle2D bounds = stroke.getBounds();
		
		return (direction == UP || direction == DOWN || bounds.getWidth() > bounds.getHeight());
	}
	
	Vector <VoiceLine> intersectingVoiceLines = null;
	VoiceLine closeLine = null, overLine = null, underLine = null;
	
	public VoiceLine getClosestVoiceLine(){
		return closeLine;
	}
	
	public VoiceLine getOverVoiceLine(){
		return overLine;
	}
	
	public VoiceLine getUnderVoiceLine(){
		return underLine;
	}
	
	public Vector <VoiceLine> getIntersectedVoiceLines(){
		return intersectingVoiceLines;
	}
	
	public Vector <VoiceLine> getCloseVoiceLines(){
		if(intersectingVoiceLines != null && !intersectingVoiceLines.isEmpty()) return intersectingVoiceLines;
		else {
			Vector<VoiceLine> v = new Vector<VoiceLine>();
			v.add(closeLine); 
			return v;
		}
	}
	
	public PolyStaff getClosestPolyStaff(){
		return closeLine.getPolyStaff();
	}
	
	public PolyStaff getOverPolyStaff(){
		if(overLine != null && overLine.isLastLineInStaff())
			return overLine.getPolyStaff();
		else return null;
	}
	
	public PolyStaff getUnderPolyStaff(){
		if(underLine != null && underLine.isFirstLineInStaff())
			return underLine.getPolyStaff();
		else return null;
	}
	
	
	public ScoreVerticalPointer getVerticalStartPointer(PolyStaff polyStaff){
		return new ScoreVerticalPointer(polyStaff, getStroke().getBounds().getMinX());
	}
	
	public ScoreVerticalPointer getVerticalEndPointer(PolyStaff polyStaff){
		return new ScoreVerticalPointer(polyStaff, getStroke().getBounds().getMaxX());
	}
	
	
	
	/////////
	
	public void setScopeConstraint(ScopeConstraint constraint){
		this.constraint = constraint;
	}
	

	public Collection<Segment> getIntersectedSegments(){
		return null;
	}

	public boolean intersects(){
		return intersectingVoiceLines != null && !intersectingVoiceLines.isEmpty();
	}
	
	/*public void setIntersectsStaff(boolean intersects) {
		this.intersects = intersects;
		
	}*/

	public void setIntersected(Vector<VoiceLine> intersectinglines) {
		intersectingVoiceLines = intersectinglines;
		
	}

	public void setCloseLine(VoiceLine closeLine) { 
		this.closeLine = closeLine;
		
	}

	public void setOverLine(VoiceLine overLine) {
		this.overLine = overLine;
		
	}

	public void setUnderLine(VoiceLine underLine) {
		this.underLine = underLine;
		
	}

	public boolean isScoreDelimiter() {
		if(isHorizontal()) return false;
		
		
		if(closeLine.getMaxY() > stroke.getMaxY()){
			if(stroke.getMaxY() - closeLine.getMinY() > closeLine.getHeight()/2) return true;
			else return false;
		}
		else {
			if(closeLine.getMaxY() - stroke.getMinY() > closeLine.getHeight()/2) return true;
			else return false;
		}		
	}

	
	GraphVoiceLine graphline = null;
	public void setGraphStaff(GraphVoiceLine voiceLine) {
		graphline = voiceLine;
		closeLine = voiceLine;
	}
	
	public GraphVoiceLine getGraphline(){
		return graphline;
	}
	
}
