package inria.paperoles.gestures.recognition;

import java.awt.geom.AffineTransform;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.List;
import java.util.Vector;

import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.SingleInkStroke;
import inria.paperoles.gestures.definition.GestureDefinition;
import inria.paperoles.grammar.GroupingGesture;
import inria.paperoles.grammar.Token;
import inria.paperoles.ui.GesturePath;

import org.sigtec.ink.Note;
import org.sigtec.ink.Point;
import org.sigtec.ink.Trace;
import org.ximtec.igesture.algorithm.AlgorithmException;
import org.ximtec.igesture.algorithm.rubine.RubineAlgorithm;
import org.ximtec.igesture.configuration.Configuration;
import org.ximtec.igesture.core.GestureClass;
import org.ximtec.igesture.core.GestureSample;
import org.ximtec.igesture.core.GestureSet;
import org.ximtec.igesture.core.Result;
import org.ximtec.igesture.core.ResultSet;
import org.ximtec.igesture.core.SampleDescriptor;


import papertoolkit.pen.PenSample;

public class ParameterRecognitionManager {

	GestureSet gestureSet;
	AdaptedRubineAlgorithm rubineAlg;
	
	// Directory where default class files are located
	public final static String DIRPATH = "gestures/"; 
	
	public ParameterRecognitionManager(){
		gestureSet = new GestureSet();
		
		rubineAlg = new AdaptedRubineAlgorithm();
		
		loadDefaultGestures();
	}
	
	
	// We initiate the algorithm with a gesture class so that it is not empty
	// We use the closed selector as the default fake gesture in the gesture set.
	private void loadDefaultGestures(){
		File directory = new File(DIRPATH);
		File[] gesturefiles = directory.listFiles();
		
		for(int i=0; i<gesturefiles.length; ++i){ 
			if(!gesturefiles[i].getName().endsWith("gst")) continue;
			
			GestureClass gestureClass = new FakeGestureClass();
			
			BufferedReader input;		
			try {
				input = new BufferedReader(new FileReader(gesturefiles[i]));
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				continue;
			}
	
			SampleDescriptor descriptor = new SampleDescriptor();
			
			Trace trace = new Trace();
			
			String str;
			try {
				int j = 0;
				while((str = input.readLine()) != null && !str.isEmpty()){		
					if(str.contains(";")){
						Note note = new Note();
						note.add(trace); 
						GestureSample gsample = new GestureSample("s" + i + "" + (j++), note);
						descriptor.addSample(gsample);			
						
						trace = new Trace();
						continue;
					}
							
					String[] coords = str.split(","); 
					trace.add(new Point(new Integer(coords[0]).intValue(), new Integer(coords[1]).intValue(), 
							new Long(coords[2]).longValue(), 1));
				}
			
				input.close();
			} catch (NumberFormatException e) {
				e.printStackTrace();
				continue;
			} catch (IOException e) {
				e.printStackTrace();
				continue;
			}	

			gestureClass.addDescriptor(descriptor);
			gestureSet.addGestureClass(gestureClass);
		}
		
	}

	public void loadClasses(Vector<GeneralizedGestureClass> classes){
		boolean changed = false;
		
		for(GeneralizedGestureClass gclass:classes)
			if(gclass.getDefinition().isCircled()){
				gestureSet.addGestureClass(gclass);
				changed = true;
			}
		
		try {
			if(changed) initRecognizer();
		} catch (AlgorithmException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	
	public void removeGestureClass(GeneralizedGestureClass gestureClass){
		gestureSet.removeGestureClass(gestureClass);
		
		try {
			initRecognizer();
		} catch (AlgorithmException e) {
	
		}
	}
	
	// the definition attribute contains context information about the gesture
	public GeneralizedGestureClass createGestureClass(GeneralInkStroke stroke, GestureDefinition definition) {
			
		Note note = transformStroke(stroke);
	
		GeneralizedGestureClass gestureClass = new GeneralizedGestureClass(definition);
		
		SampleDescriptor descriptor = new SampleDescriptor();
	
		GestureSample gsample = new GestureSample(definition.getName() + "1", note);
		descriptor.addSample(gsample);		
		
		//add also a slightly zoomed version of the stroke
		gsample = new GestureSample(definition.getName() + "2", transformStroke(stroke.getScaledStroke(1.1, 1.1)));
		descriptor.addSample(gsample);
		
		// a slightly shrunk version
		gsample = new GestureSample(definition.getName() + "3", transformStroke(stroke.getScaledStroke(.9, 9)));
		descriptor.addSample(gsample);		
		
		// an other shrunk version
//		gsample = new GestureSample(definition.getName() + "4", transformStroke(stroke.getScaledStroke(.9, .7)));
//		descriptor.addSample(gsample);			
		
		gsample = new GestureSample(definition.getName() + "5", transformStroke(stroke.getRotatedStroke(Math.PI/40)));
		descriptor.addSample(gsample);
		
		gsample = new GestureSample(definition.getName() + "6", transformStroke(stroke.getRotatedStroke(-Math.PI/40)));
		descriptor.addSample(gsample);
		
		gestureClass.addDescriptor(descriptor);
		gestureSet.addGestureClass(gestureClass);
		
		try {
			initRecognizer();
		} catch (AlgorithmException e) {
			gestureSet.removeGestureClass(gestureClass);
			
			try {
				initRecognizer();
			} catch (AlgorithmException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			
			return null;
		}
		
		return gestureClass;
		//////////
	}
	
	public void addInstanceToClass(GeneralInkStroke stroke, GeneralizedGestureClass gestureClass){
		Note note = transformStroke(stroke);
		
		//SampleDescriptor descriptor = new SampleDescriptor();
		SampleDescriptor descriptor = (SampleDescriptor) gestureClass.getDescriptors().get(0);
		GestureDefinition definition = gestureClass.getDefinition();
		
		GestureSample gsample = new GestureSample(definition.getName() + "1", note);
		descriptor.addSample(gsample);		
		
		//add also a slightly zoomed version of the stroke
		gsample = new GestureSample(definition.getName() + "2", transformStroke(stroke.getScaledStroke(1.1, 1.1)));
		descriptor.addSample(gsample);
		
		// a slightly shrunk version
		gsample = new GestureSample(definition.getName() + "3", transformStroke(stroke.getScaledStroke(.9, 9)));
		descriptor.addSample(gsample);		
		
		// an other shrunk version
//		gsample = new GestureSample(definition.getName() + "4", transformStroke(stroke.getScaledStroke(.9, .7)));
//		descriptor.addSample(gsample);			
		
		gsample = new GestureSample(definition.getName() + "5", transformStroke(stroke.getRotatedStroke(Math.PI/40)));
		descriptor.addSample(gsample);
		
		gsample = new GestureSample(definition.getName() + "6", transformStroke(stroke.getRotatedStroke(-Math.PI/40)));
		descriptor.addSample(gsample);
		
	//	gestureClass.addDescriptor(descriptor);
	//	gestureSet.addGestureClass(gestureClass);
		
		try {
			initRecognizer();
		} catch (AlgorithmException e) {
		//	gestureSet.removeGestureClass(gestureClass);
			

			
		}
		
	}
	
	private void initRecognizer() throws AlgorithmException{
		Configuration config = new Configuration();
		config.addGestureSet(gestureSet);
		
		rubineAlg.init(config);
	}
	
	
	ResultSet completeResult;
	private ResultSet classifyStroke(GeneralInkStroke stroke) throws AlgorithmException{

		Note note = transformStroke(stroke);
	
		ResultSet result = rubineAlg.recognise(note);
		completeResult = rubineAlg.getCompleteResultSet();	
		return result;
	}
	
	public ResultSet getCompleteResult(){
		return completeResult;
	}
	
	
	private Note transformStroke(GeneralInkStroke stroke){
		Note note = new Note();
		
		///???
		if(stroke == null) return null;
		
		for(SingleInkStroke singleStroke:stroke.getStrokes()){
			Trace trace = new Trace();
			
			for(PenSample sample:singleStroke.getSamples()){
				Point point = new Point(sample.getX(), sample.getY(), sample.getTimestamp());
				
				trace.add(point);
			}
			
			note.add(trace);
		}
		
		return note;
	}


	public void recognizeGesture(GesturePath gesture) throws AlgorithmException {
		if(gestureSet.size() < 2){
			gesture.assignToGestureClass(null, 0, false);
			gesture.addPossibleClasses(null);
			return;
		}
		
		RecognitionResult result = (RecognitionResult)classifyStroke(((GroupingGesture)gesture.getToken()).getGroup().combineStroke()).getResult();

		if(result!=null){
			GeneralizedGestureClass gclass = (GeneralizedGestureClass)result.getGestureClass();
				
			if(gclass != null && !(gclass instanceof FakeGestureClass))
				gesture.assignToGestureClass(gclass, result.getDistance(), false);
			else gesture.assignToGestureClass(null, 0, false);
		}
		else gesture.assignToGestureClass(null, 0, false);
		
		Vector<GeneralizedGestureClass> classes = new Vector<GeneralizedGestureClass>();
	
		if(completeResult != null)
			for(Result res:completeResult.getResults())	
				classes.addElement((GeneralizedGestureClass)res.getGestureClass());
	
		gesture.addPossibleClasses(classes);
	}




	
}
