package inria.paperoles.gestureslearn;

import javax.swing.JOptionPane;
import javax.swing.JDialog;
import javax.swing.JTextField;

import org.sigtec.ink.Point;

import java.beans.*; //property change stuff
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;
import java.awt.*;
import java.awt.event.*;

/* 1.4 example used by DialogDemo.java. */
class GestureDialog extends JDialog implements ActionListener, PropertyChangeListener {
    private String typedText = null;
    private JTextField textField;

    private String magicWord;
    private JOptionPane optionPane;

    private String btnString1 = "Enter";
    private String btnString2 = "Cancel";

	public final static String DIRPATH = "gestures/"; 
    
    private Vector<Point> points;
    /**
     * Returns null if the typed string was invalid;
     * otherwise, returns the string as the user entered it.
     */
    public String getValidatedText() {
        return typedText;
    }

    /** Creates the reusable dialog. */
    public GestureDialog(Frame aFrame, Vector<Point> points2) {
        super(aFrame, true);

        this.points = points2;
        setTitle("Gesture Recording");

        textField = new JTextField(10);

        //Create an array of the text and components to be displayed.
        String msgString = "Type the name (class) of the gesture.";

        Object[] array = {msgString, textField};

        //Create an array specifying the number of dialog buttons
        //and their text.
        Object[] options = {btnString1, btnString2};

        //Create the JOptionPane.
        optionPane = new JOptionPane(array,
                                    JOptionPane.QUESTION_MESSAGE,
                                    JOptionPane.YES_NO_OPTION,
                                    null,
                                    options,
                                    options[0]);

        //Make this dialog display it.
        setContentPane(optionPane);

        //Handle window closing correctly.
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                /*
                 * Instead of directly closing the window,
                 * we're going to change the JOptionPane's
                 * value property.
                 */
                    optionPane.setValue(new Integer(
                                        JOptionPane.CLOSED_OPTION));
            }
        });

        //Ensure the text field always gets the first focus.
        addComponentListener(new ComponentAdapter() {
            public void componentShown(ComponentEvent ce) {
                textField.requestFocusInWindow();
            }
        });

        //Register an event handler that puts the text into the option pane.
        textField.addActionListener(this);

        //Register an event handler that reacts to option pane state changes.
        optionPane.addPropertyChangeListener(this);
    }

    /** This method handles events for the text field. */
    public void actionPerformed(ActionEvent e) {
        optionPane.setValue(btnString1);
    }

    /** This method reacts to state changes in the option pane. */
    public void propertyChange(PropertyChangeEvent e) {
        String prop = e.getPropertyName();

        if (isVisible()
         && (e.getSource() == optionPane)
         && (JOptionPane.VALUE_PROPERTY.equals(prop) ||
             JOptionPane.INPUT_VALUE_PROPERTY.equals(prop))) {
            Object value = optionPane.getValue();

            if (value == JOptionPane.UNINITIALIZED_VALUE) {
                //ignore reset
                return;
            }

          
            //Reset the JOptionPane's value.
            //If you don't do this, then if the user
            //presses the same button next time, no
            //property change event will be fired.
            optionPane.setValue(
                    JOptionPane.UNINITIALIZED_VALUE);

            if (btnString1.equals(value)) {
                
            	typedText = textField.getText();
                
                try {
					registerGesture(typedText.toLowerCase()+".gst");
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
                clearAndHide();
                
            } else { //user closed dialog or clicked cancel
            	///////////
            	
                typedText = null;
                clearAndHide();
            }
        }
    }
//Todo: 
    private void registerGesture(String filename) throws IOException {
    	BufferedWriter output = new BufferedWriter(new FileWriter(DIRPATH + filename, true));
		
    	for(Point point:points){
    		output.write((int)point.x + "," + (int)point.y + "," + point.getTimestamp() +"\n");
    	}
    	
    	output.write(";\n");
		output.close();		
    	
	}

	/** This method clears the dialog and hides it. */
    public void clearAndHide() {
        textField.setText(null);
        setVisible(false);
    }
}
