package inria.paperoles.grammar;

import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.ui.GesturesCanvas;

import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Vector;

public class ComplexGesture extends Token{

	Vector<Token> tokens;
	
//	SortedSet<Token> tokens;
	
	public ComplexGesture() {
		super(null, null);
		
		tokens = new Vector<Token>();
	}

	public ComplexGesture(GeneralInkStroke stroke, GesturesCanvas gcanvas) {
		super(stroke, gcanvas);
		
		tokens = new Vector<Token>();
	}
	
	public ComplexGesture(Token token) {
		this();
		
		addToken(token);
	}
	
	public void addToken(Token token){
		if(!tokens.contains(token)) tokens.add(token);
		
		for(GesturesCanvas canvas:token.gcanvases)
			gcanvases.add(canvas);
		
		token.setParent(this);
	}
	
	public void removeToken(Token token){
		tokens.remove(token);
		GesturesCanvas canvas = token.getCanvas();
		
		boolean toremove = true;
		for(GesturesCanvas gcanvas:gcanvases)
			if(gcanvas == canvas){
				toremove = false;
				break;
			}
		
		if(toremove) gcanvases.remove(canvas);
	}
	
	
	public Vector<Token> getSubTokens(){
		return tokens;
	}
	
	public boolean isLeaf(){
		return false;
	}

	public boolean closeTo(Token token, int maxDist) {
		if(maxDist < 0) return true;
		
		for(Token t:tokens)
			if(t.closeTo(token, maxDist)) return true;
	
		return false;
	}
	
	public boolean intersects(Line2D line) {
		for(Token t:tokens)
			if(t.intersects(line)) return true;
		
		return false;
	}
	
	public Token getIntersectedToken(Collection<Token> tokens) {	
		for(Token t:this.tokens){ 	
			if(t == this) continue;
			Token token = t.getIntersectedToken(tokens);
			if(token!=null)	return token;
		}
		
		return null;
	}
	
	public Token intersectsWith(Token token) {
		for(Token t:tokens)
			if(t.intersectsWith(token)!=null) return this;
		
		return null;
	}	
	
	public boolean isRecognizable() {
		return false;
	}
	
	/*
	public String toString(){
		String str = super.toString();
		for(Token t:tokens)
			str +="\n - " + t.toString();
		
		return str;
	}*/
}
