package inria.paperoles.grammar;

import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.SingleStrokeAnalysisResult;
import inria.paperoles.scoping.ScopeInstance;
import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.scoremodel.Voice;
import inria.paperoles.scoremodel.VoiceLine;

import java.awt.Shape;
import java.awt.geom.Area;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Vector;

public class ParenthesizedToken extends Token{
	
	ParenthesisToken leftDelimiter;
	ParenthesisToken rightDelimiter;
	
	public ParenthesizedToken (ParenthesisToken delimiter1, ParenthesisToken delimiter2) {
		super(null, null);
		
		gcanvases.add(delimiter1.getCanvas());
		gcanvases.add(delimiter2.getCanvas());
		
		delimiter1.setParent(this);
		delimiter2.setParent(this);
		
		if(delimiter1.getFirstStroke().getMinX() < delimiter2.getFirstStroke().getMinX()){
			leftDelimiter = delimiter1;
			rightDelimiter = delimiter2;
		}
		else {
			rightDelimiter = delimiter1;
			leftDelimiter = delimiter2;
		}
	
		types = new int[1];
		types[0] = Scope.INSIDE;
	}
		
	public void removeDependantGestures(){//System.err.println(" &&& " + this);
		rightDelimiter.removeDependantGestures();
		leftDelimiter.removeDependantGestures();
	}
	
	public boolean isPaired(){
		return leftDelimiter != null;
	}
	
	// return the last in time stroke...
	public GeneralInkStroke getFirstStroke(){
		if(leftDelimiter.getFirstStroke().getFirstTimestamp() < rightDelimiter.getFirstStroke().getFirstTimestamp())
		return leftDelimiter.getFirstStroke();
		else return rightDelimiter.getFirstStroke();
	}
	
	public GeneralInkStroke getLastStroke(){
		if(leftDelimiter.getFirstStroke().getFirstTimestamp() < rightDelimiter.getFirstStroke().getFirstTimestamp())
			return rightDelimiter.getFirstStroke();
			else return leftDelimiter.getFirstStroke();
	}
	
	public ParenthesisToken getLeft(){
		return leftDelimiter;
	}
	
	public ParenthesisToken getRight(){
		return rightDelimiter;
	}

	
	public Shape getEnclosedArea() { 
		double x1 = leftDelimiter.getFirstStroke().getShape().getBounds().getMinX();
		double y1 = leftDelimiter.getFirstStroke().getShape().getBounds().getMinY();
		
		double x2 = rightDelimiter.getFirstStroke().getShape().getBounds().getMaxX();
		double y2 = rightDelimiter.getFirstStroke().getShape().getBounds().getMaxY();
		
		return new Rectangle2D.Double(x1, y1, x2 - x1, y2 - y1);
	}
	
	public boolean isLeaf(){
		return false;
	}	
	
	public Collection<Token> getSubTokens(){
		Vector<Token> subTokens = new Vector<Token>();
		subTokens.add(leftDelimiter);
		subTokens.add(rightDelimiter);
		
		return subTokens;
	}
	
	public Scope getScope(int type){
		if(type!= Scope.INSIDE) return null;
		
		Area area = new Area(getEnclosedArea());
		area.subtract(leftDelimiter.getLeftOuterSpace());
		area.subtract(rightDelimiter.getRightOuterSpace());
		
		return new WrittenScope(leftDelimiter.getCanvas(), area);
	}
	
	public boolean intersects(Line2D line) {
	
		if(leftDelimiter.intersects(line) || rightDelimiter.intersects(line)) return true;
		
		return false;
	}
	
	public Token intersectsWith(Token token) {
		if(leftDelimiter.intersectsWith(token)!=null 
				|| rightDelimiter.intersectsWith(token)!=null) return this;
		
		return null;
	}	
	
	public boolean isRecognizable() {
		return false;
	}
}
