package inria.paperoles.grammar;

import java.awt.Polygon;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.SingleInkStroke;
import inria.paperoles.gestures.SingleStrokeAnalysisResult;
import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.scoremodel.ScoreVerticalPointer;
import inria.paperoles.ui.GesturesCanvas;

public class ScopeDelimiter extends Token{

	ScoreVerticalPointer pointer1, pointer2;
	
	ScorePointer scorePointer;
	
	int pigTailIndex = -1;
	
	GeneralInkStroke strokeNoTail;
	
	public ScopeDelimiter(GeneralInkStroke stroke, GesturesCanvas gcanvas, boolean outOfScore) {
		super(stroke, gcanvas);
		
		pigTailIndex = stroke.getPigTailIndex();
		if(pigTailIndex > 0) strokeNoTail = new SingleInkStroke(stroke, pigTailIndex);
		else strokeNoTail = stroke;
	}
	
	public ScopeDelimiter(GeneralInkStroke stroke, GesturesCanvas gcanvas) {
		super(stroke, gcanvas);
	
		SingleStrokeAnalysisResult analysisResult = getFirstStroke().getAnalysisResult();	
		pointer1 = analysisResult.getVerticalStartPointer(analysisResult.getClosestPolyStaff());
		pointer2 = analysisResult.getVerticalEndPointer(analysisResult.getClosestPolyStaff());

		pigTailIndex = stroke.getPigTailIndex();
		if(pigTailIndex > 0) strokeNoTail = new SingleInkStroke(stroke, pigTailIndex);
		else strokeNoTail = stroke;
		
		scorePointer = new ScorePointer(stroke, pointer1, false);
	}

	
	
	public boolean matches(ScopeDelimiter delimiter){
		if(getClass() != delimiter.getClass()) return true;
		else return false;
	}
	
	public ScoreVerticalPointer getStartPointer(){
		return pointer1;
	}
	
	public ScoreVerticalPointer getEndPointer(){
		return pointer2;
	}
	

	public ScopingPair merge(ScopeDelimiter delimiter, SymbolTable table){
		ScopingPair pair = new ScopingPair(this, delimiter);
		
		table.tokens.remove(delimiter);
		table.tokens.add(table.tokens.indexOf(this), pair);
		table.tokens.remove(this);
		
		return pair;
	}

	
	public ScorePointer getScorePointer(){
		return scorePointer;
	}
	
	public Area getLeftOuterSpace(){
		PolyStaff poly = stroke.getAnalysisResult().getClosestPolyStaff();
		
		boolean looksRight = !stroke.getAnalysisResult().isLeft();
		
		
		Rectangle2D rect = strokeNoTail.getBounds();
		double y = Math.min(rect.getMinY(), poly.getMinY() - 40);
		rect.setRect(rect.getMinX(), y, rect.getWidth(), Math.max(rect.getMaxY(), poly.getMaxY() + 40) - y);
		
		Point2D p1 = strokeNoTail.getFirstPoint();
		Point2D p2 = strokeNoTail.getLastPoint();
		
		Polygon polygon = new Polygon();
		polygon.addPoint((int)p1.getX(), (int)p1.getY());
		polygon.addPoint((int)p2.getX(), (int)p2.getY());
	
		
		if(p1.getX() > p2.getX() && p1.getY() < p2.getY()
				|| p1.getX() < p2.getX() && p1.getY() > p2.getY()){
			polygon.addPoint((int)p2.getX(), looksRight?(int)rect.getMaxY():(int)rect.getMinY());
			polygon.addPoint((int)p1.getX(), looksRight?(int)rect.getMaxY():(int)rect.getMinY());
		}
		else {
			polygon.addPoint((int)p2.getX(), looksRight?(int)rect.getMinY():(int)rect.getMaxY());
			polygon.addPoint((int)p1.getX(), looksRight?(int)rect.getMinY():(int)rect.getMaxY());
		}

		Area area = new Area(rect);
		
		Area fill = new Area(strokeNoTail.getShape());
		fill.add(new Area(polygon));
		
		if(looksRight){
			area.subtract(fill);
			return area;
		}
		else return fill;
	}	
	
	
	public Area getRightOuterSpace(){
		PolyStaff poly = stroke.getAnalysisResult().getClosestPolyStaff();
		boolean looksLeft = stroke.getAnalysisResult().isLeft();
		
		Rectangle2D rect = strokeNoTail.getBounds();
		double y = Math.min(rect.getMinY(), poly.getMinY() - 40);
		rect.setRect(rect.getMinX(), y, rect.getWidth(), Math.max(rect.getMaxY(), poly.getMaxY() + 40) - y);		
		
		Point2D p1 = strokeNoTail.getFirstPoint();
		Point2D p2 = strokeNoTail.getLastPoint();
		
		Polygon polygon = new Polygon();
		polygon.addPoint((int)p1.getX(), (int)p1.getY());
		polygon.addPoint((int)p2.getX(), (int)p2.getY());
			
		if(p1.getX() > p2.getX() && p1.getY() < p2.getY()
				|| p1.getX() < p2.getX() && p1.getY() > p2.getY()){
			polygon.addPoint((int)p2.getX(), looksLeft?(int)rect.getMinY():(int)rect.getMaxY());
			polygon.addPoint((int)p1.getX(), looksLeft?(int)rect.getMinY():(int)rect.getMaxY());
		}
		else {
			polygon.addPoint((int)p2.getX(), looksLeft?(int)rect.getMaxY():(int)rect.getMinY());
			polygon.addPoint((int)p1.getX(), looksLeft?(int)rect.getMaxY():(int)rect.getMinY());
		}
		
		Area area = new Area(rect);
		
		Area fill = new Area(strokeNoTail.getShape());
		fill.add(new Area(polygon));
		
		if(looksLeft){
			area.subtract(fill);
			return area;
		}
		else return fill;
	}
	
	public Token getContainerToken() {
		if(getParent()!=null && !(getParent() instanceof ComplexGesture)) return getParent();
		return this;
	}
	
	public boolean hasPigTail(){
		return (pigTailIndex > 0);
	}

	public String toString(){	
		
		return "Vertical Line";
	}
	
}
