package inria.paperoles.grammar;

import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.SingleStrokeAnalysisResult;
import inria.paperoles.scoremodel.Chord;
import inria.paperoles.scoremodel.EndScore;
import inria.paperoles.scoremodel.Measure;
import inria.paperoles.scoremodel.ScorePart;
import inria.paperoles.scoremodel.ScoreVerticalPointer;
import inria.paperoles.scoremodel.Segment;
import inria.paperoles.scoremodel.VoiceLine;
import inria.paperoles.scoremodel.WavePoint;

import java.awt.Rectangle;
import java.awt.geom.Area;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.Vector;

public class ScorePointer extends Scope {

	Collection<ScorePart> scoreParts;
	
	public ScorePointer(ScorePart scorePart){
		super(scorePart.getCanvas(), Scope.POINTER);
		scoreParts = new Vector<ScorePart>();
		scoreParts.add(scorePart);
		
		createShape();
	}
	
	
	public ScorePointer(Vector<ScorePart> scoreParts){
		super(scoreParts.firstElement().getCanvas(), Scope.POINTER);
		this.scoreParts = scoreParts;
		
		createShape();
	}
	
	public ScorePointer(GeneralInkStroke stroke, ScoreVerticalPointer pointer, boolean intersectedOnly){		
		super(pointer.getMeasure().getPolyStaff().getSkeleton().getCanvas(), Scope.POINTER);
	
		
		SingleStrokeAnalysisResult result = stroke.getAnalysisResult();
		
		Vector<VoiceLine> vlines = result.getIntersectedVoiceLines();
		
		if(vlines == null || vlines.isEmpty()){
			if(intersectedOnly){
				scoreParts =  null;
				return;
			}
			else vlines = result.getCloseVoiceLines();
		}
		scoreParts = new Vector<ScorePart>();
		
		Point2D endPoint = stroke.getLastPoint();
		
		double strokeMinY = Math.min(stroke.getFirstPoint().getY(), endPoint.getY());
		double strokeMaxY = Math.max(stroke.getFirstPoint().getY(), endPoint.getY());
		
		double strokeMinX, strokeMaxX;
		
		if(stroke.getFirstPoint().getY() == strokeMinY){
			strokeMinX = stroke.getFirstPoint().getX();
			strokeMaxX = endPoint.getX();
		}
		else {
			strokeMinX = endPoint.getX();
			strokeMaxX = stroke.getFirstPoint().getX();
		}
		
		Line2D overExtensionLine = new Line2D.Double(strokeMinX, strokeMinY, strokeMinX, strokeMinY - 80);
		Line2D underExtensionLine = new Line2D.Double(strokeMaxX, strokeMaxY, strokeMaxX, strokeMaxY + 80);
		
		Measure measure, measurelast = null;
		
		for(VoiceLine vline:vlines){
			boolean noteFound = false;
			for(measure = pointer.getMeasure(); measure!=null && !noteFound && measure.getPolyStaff() == vline.getPolyStaff(); measure = measure.getNextMeasure()){
				Segment segment = measure.getSegment(vline.getVoice());
			
				measurelast = measure;
				for(Chord chord: segment.getChords()){
					Line2D line = new Line2D.Double(chord.getCenterX(), chord.getCenterY(), vline.getMaxX(), chord.getCenterY());
					
					if(stroke.intersects(line) || line.intersectsLine(overExtensionLine) || line.intersectsLine(underExtensionLine)) continue;
					else {
						noteFound = true;
						scoreParts.add(chord);
						break;
					}
						
				}
			}
				
			if(!noteFound) {
				if(measure == null)scoreParts.add(new EndScore(measurelast.getSegment(vline.getVoice())));
				else{
					Measure measureNext = measure.getNextMeasure();
					if(measureNext == null) scoreParts.add(new EndScore(measure.getSegment(vline.getVoice())));
					else scoreParts.add(measure.getSegment(vline.getVoice()));
				}
			}	
		}
		
		createShape();
	}
	
	private void createShape(){
		shape = new Rectangle(0,0,0,0);
		/*
		shape = new Area();
		for(ScorePart scorePart:scoreParts)
			((Area) shape).add(new Area(scorePart.getPointerShape()));
		*/
	}
	
	public Collection<ScorePart> getPointer(){
		return scoreParts;
	}


	@Override
	public String toStringRepresentation() {
		String s = "";
		
		for(ScorePart spart:scoreParts)
			s+=" (" + spart + ")";
		
		return s;
	}

}
