package inria.paperoles.grammar;

import java.awt.Shape;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.Vector;

public class SymbolTable {

	Vector<Token> oldtokens;
	Vector<Token> tokens;
	
	public SymbolTable(){
		tokens = new Vector<Token>();
		oldtokens = new Vector<Token>();
	}
	
	public SymbolTable(SymbolTable table){
		tokens = new Vector<Token>();
		oldtokens = table.oldtokens;
	}
	
	
	public void addToken(Token token){
		tokens.add(token);
		
		token.setSymbolTable(this);
	}
	
	public Token getAssociatedToken(Token token){
		for(int i = tokens.size() - 1; i>=0; --i){
			Token t = tokens.elementAt(i);
			if(token.isPairWith(t)) return t;
		}
				
		return null;
	}
	
	public void renew(){
		oldtokens.addAll(tokens);
		tokens.removeAllElements();
	}
	
	public Collection<Token> getTokens(){
		return tokens;
	}
	
	public Collection<Token> getOldTokens(){
		return oldtokens;
	}
	
	public void removeToken(Token token) {
		tokens.remove(token);
		oldtokens.remove(token);
		
		Collection<Token> subtokens = token.getSubTokens();
		if(subtokens == null) return;
		
		for(Token subtoken:subtokens)
			removeToken(subtoken);
	}

		
	//TODO: need to improve the accuracy of the selection
	public Collection<Token> select(Token token, Shape shape) {
		Vector<Token> selected = new Vector<Token>();

		for(int i = 0; i<tokens.size(); ++i){
			Token t = tokens.elementAt(i);			
		
			if(t.gcanvas != token.gcanvas || t == token) continue;
			
//			if(t == token 
//					|| (t instanceof SelectorToken) && ((SelectorToken)t).getSelectedToken() == token) continue; 
				
			if(t instanceof ParenthesizedToken){
				Rectangle2D rect = shape.getBounds2D();
				rect.setRect(rect.getX() - 4, rect.getY() - 4, rect.getWidth()+8, rect.getHeight() + 8);
				
				if(((ParenthesizedToken)t).getLeft().isContainedIn(rect) > .8){
					t.setRemove();
					selected.add(((ParenthesizedToken)t).getLeft());
				}
				if(((ParenthesizedToken)t).getRight().isContainedIn(rect) > .8){
					t.setRemove();
					selected.add(((ParenthesizedToken)t).getRight());
				}
			}
	/*		else if(t instanceof SelectorToken && ((SelectorToken)t).getSelectedToken() == token) continue;
			else if(t instanceof ConnectorToken && (((ConnectorToken)t).getStart() == token
					|| (((ConnectorToken)t).getEnd() == token))) continue;	
			else 
			*/
			
			double r = t.isContainedIn(shape);
	
			if(r >= .85 || r >= .5 && !(t instanceof SelectorToken) ){
				System.err.println(t + " contained in " + token);
				selected.add(t);	
			}
			
		}
		
		return selected;
	}

	public Group groupTogether(Collection<Token> stokens, SymbolTable table) {
		Group group = new Group();
		
		for(Token token:stokens){
			if(token.removed()) continue;
	
			Grouped grouped = new Grouped(token);
			group.addToken(grouped);
			grouped.setParent(group);
			
			table.removeToken(token);

			token.setRemove();
		}
			
		return group;
	}

	public String toString(){
		String str = new String("------------\n");
		for(Token token:tokens){
			str += token + "\n";
		}
		
		return str;
	}

	public void clean(){
		tokens.clear();
		oldtokens.clear();
	}
	
	public void removeTokens(Vector<Token> tokens) {
		for(Token token:tokens)
			if(this.tokens.remove(token)) continue;
	}
}
