package inria.paperoles.grammar;

import inria.paperoles.docmodel.DocSkeleton;
import inria.paperoles.docmodel.Symbol;
import inria.paperoles.gestures.ArrowStroke;
import inria.paperoles.gestures.ComplexInkStroke;
import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.SingleInkStroke;
import inria.paperoles.gestures.SingleStrokeAnalysisResult;
import inria.paperoles.operations.MatchedOperation;
import inria.paperoles.scoping.ScopeInstance;
import inria.paperoles.scoremodel.MultiplePageSkeleton;
import inria.paperoles.scoremodel.ScoreSkeleton;
import inria.paperoles.scoremodel.Voice;
import inria.paperoles.scoremodel.VoiceLine;
import inria.paperoles.scoremodel.WaveLine;
import inria.paperoles.ui.GesturesCanvas;
import inria.paperoles.ui.GlobalParameters;

import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;
import java.util.Stack;
import java.util.TreeSet;
import java.util.Vector;

import papertoolkit.pen.ink.InkStroke;

public class SyntaxAnalyzer {

	long MAX_LAG = 1700;
	int MAX_DIST = -1;
	
	MultiplePageSkeleton skeleton;
	SymbolTable symbolTable;
	
	Hashtable<Token, MatchedOperation> matchedOperations;
//	HashSet<GroupingGesture> groupings;
	
	public SyntaxAnalyzer(MultiplePageSkeleton skeleton){
		this.skeleton = skeleton;
		
		symbolTable = new SymbolTable();
		matchedOperations = new Hashtable<Token, MatchedOperation>();
		
//		groupings = new HashSet<GroupingGesture>();
	}
	
	public void recognizeSyntax(){
		
// OK, I shouln't recognize Control Points always....	
		symbolTable = recognizeRoundedScopes(recognizeTokens(symbolTable));

//		findWeakAssociations();
		findStrongAssociations();
	}
	


	public SymbolTable getSymbolTable(){
		return symbolTable;
	}
	
	/*public Set<GroupingGesture> WgetGroupings(){
		return groupings;
	}*/

	public Token analyzeNewStroke(SingleInkStroke stroke, GesturesCanvas gcanvas){
		Token token = new Token(stroke, gcanvas);
		symbolTable.addToken(token);
		
		return token;
	}
	

	
	Token token = null;
	private Token recognizeToken(Token newToken){
	
		SingleInkStroke stroke = (SingleInkStroke) newToken.getFirstStroke();
		if(stroke == null) return null;
		SingleStrokeAnalysisResult result = stroke.getAnalysisResult();
		GesturesCanvas gcanvas = newToken.getCanvas();
		
		//recognition of circles and other closed curves 
		if(result.getStrokeClass() == SingleStrokeAnalysisResult.CLOSED_SELECTION){ 
			return new ClosedSelector(stroke, gcanvas);
		} 	 	
		// recognition of other horizontal lines
		else return new Horizontal(stroke, gcanvas, (result.getGraphline() != null));
	}


	private SymbolTable recognizeTokens(SymbolTable symbolTable){
		SymbolTable table = new SymbolTable(symbolTable);	
		Token oldToken;
		
		for(Token tok:symbolTable.getTokens()){  
			tok = recognizeToken(tok);
			oldToken = tok;
			
			if(token!=null){
				if(token.removed) table.removeToken(token);
			}
			
			token = tok;
			table.addToken(token);
		}
		
		return table;
	}

	public void removeToken(Token token) {
		token.setRemove();
	}	
	
	
	public void renew(){		
		symbolTable.renew();
		
//		matchedOperations.clear();
//		matchOperations(null);
	}
	
	
	// use it to find close connections....
	private void findStrongAssociations(){
		
		for(int i = 0; i<symbolTable.getTokens().size(); ++i){ 
			token = symbolTable.tokens.elementAt(i);
			
	//		if(token instanceof GroupingGesture) groupings.add((GroupingGesture) token);
			
			Token token1, token2;
			
			if(!token.removed() && (token2 = token.getProximityToken(symbolTable.getTokens()))!=null){
				LineConnection connection = new LineConnection(token, token2);
				token.addConnection(connection);
				connection = new LineConnection(token2, token);
				token2.addConnection(connection);
					
				token.setFree(false);
				token2.setFree(false);
			}
		}
	}
	
	
	// identifies possible scopes starting from closed lines, moving to coupled vertical delimiters, single delimiters, and finally
	// horizontal lines...	
	private SymbolTable recognizeRoundedScopes(SymbolTable symbolTable){
		SymbolTable table = new SymbolTable(symbolTable);

	//TODO: see how ungrouping/grouping etc... should be implemented in relation with more complex gestures!!!!!!!!
		/// Now, I don't give any such alternative to change scope from a grouping-rounded scope to something else!!!

		for(Token token:symbolTable.getTokens()){ 
			if(token.removed()) continue;
			
			if(token instanceof ClosedSelector){	
				GeneralInkStroke stroke = token.getFirstStroke();

				Collection<Token> stokens = symbolTable.select(token, stroke.getShape());				
				if(!stokens.isEmpty()){ 	
					
		//			token.setActiveScope(new WrittenScope(token.getCanvas(), stokens, stroke.getShape()));

					Group group = symbolTable.groupTogether(stokens, table);
					
					GroupingGesture complexGesture = new GroupingGesture(token, group);
					
					table.addToken(complexGesture);
					
		//			complexGesture.detectScope();
				}
				else {
					table.addToken(new Horizontal(token.stroke, token.gcanvas, false));
				}
			}
			else table.addToken(token);
		}
		
		return table;
	}
	

	public void clean() {
		symbolTable.clean();
		matchedOperations.clear();
	}


	
}
