package inria.paperoles.grammar;

import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Line2D.Double;
import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;
import java.util.Set;
import java.util.SortedMap;
import java.util.Vector;

import de.sciss.net.OSCBundle;
import de.sciss.net.OSCMessage;

import papertoolkit.pen.PenSample;

import inria.paperoles.docmodel.Symbol;
import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.SingleStrokeAnalysisResult;
import inria.paperoles.gestures.recognition.GeneralizedGestureClass;
import inria.paperoles.operations.MatchedOperation;
import inria.paperoles.operations.Operation;
import inria.paperoles.operations.OperationArgument;
import inria.paperoles.scoping.ScopeInstance;
import inria.paperoles.scoremodel.Note;
import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.scoremodel.ScorePart;
import inria.paperoles.scoremodel.ScoreSkeleton;
import inria.paperoles.scoremodel.Voice;
import inria.paperoles.scoremodel.VoiceLine;
import inria.paperoles.scoremodel.WaveLine;
import inria.paperoles.scoremodel.WavePoint;
import inria.paperoles.ui.GesturePath;
import inria.paperoles.ui.GesturesCanvas;
import inria.paperoles.ui.GlobalParameters;
import inria.paperoles.ui.OutputWindow;

public class Token{
	public static final int WAVE_LINE = 1;

	GeneralInkStroke stroke;
	
	Scope activescope;
	AlternativeScopes scopes;
	
	boolean removed = false;
	
	boolean isAlone = true;
	SymbolTable stable = null;
	
	Set<GesturesCanvas> gcanvases;
	GesturesCanvas gcanvas;
	
	Token parent = null;
	
	int[] types;
	
	Connections connections;
	
	public Token(GeneralInkStroke stroke, GesturesCanvas gcanvas){
		gcanvases = new HashSet<GesturesCanvas>();
		this.gcanvas = gcanvas;
	
		if(gcanvas != null) gcanvases.add(gcanvas);

		this.stroke = stroke; 
		if(stroke!=null) stroke.setToken(this);
		
		scopes = new AlternativeScopes();
		types = null;
		
		if(connections == null) connections = new Connections(this);
	}
	
	
	
	public Collection<Token> getLinkedTokens(){
		if(connections == null) return null;
		else return connections.getConnectedTokens();
	}
	
	
	public void addConnection(Connection connection){
		connections.addConnection(connection);
	}
	
	public void removeConnection(Connection connection){
		if(connections != null)
			connections.removeConnection(connection);
	}
	
	public void setParent(Token token){
		this.parent = token;
	}
	
	public Token getParent(){
		return parent;
	}
	
	public void flatten(){
		parent = null;
	}
	
	public Token getRoot(){
		if(parent == null) return this;
		return parent.getRoot();
	}
	
	Hashtable<GesturesCanvas, GesturePath> gestures;
	public void addGesture(GesturePath path){
		addGesture((GesturesCanvas)path.getParent(), path);
	}
	
	public void addGesture(GesturesCanvas canvas, GesturePath path){
		if(gestures == null) gestures = new Hashtable<GesturesCanvas, GesturePath>();

		gestures.put(canvas, path);
	}
	
	
	public GesturePath getGesturePath(GesturesCanvas canvas){
		if(gestures == null) return null;
		return gestures.get(canvas);
	}
	
	public Collection<GesturePath> getOtherPaths(GesturePath path){
		if(gestures == null || gestures.isEmpty()) return new HashSet<GesturePath>();
		
		HashSet<GesturePath> other = new HashSet<GesturePath>(gestures.values());
		other.remove(path);
		
		return other;
	}
	
	public Collection<GesturePath> getAllPaths(){
		if(gestures == null || gestures.isEmpty()) return new HashSet<GesturePath>();
		
		return gestures.values();
	}
	
	public boolean isPairWith(Token token) {
		return false;
	}
	
	public Collection<GesturesCanvas> getCanvases(){
		return gcanvases;
	}
	
	public GesturesCanvas getCanvas(){
		return gcanvas;
	}
	
	public boolean hasStrokeIn(GesturesCanvas canvas){
		return gcanvases.contains(canvas);
	}
	
	public boolean hasSkopeIn(GesturesCanvas canvas){
		if(activescope == null) return false;
		return activescope.inGesturesCanvas(canvas);
	}
	
	public GeneralInkStroke getLastStroke(){
		return stroke;
	}
	
	public GeneralInkStroke getFirstStroke(){
		return stroke;
	}
	
	public void setActiveScope(Scope scope){
		if(activescope == scope) return;
		
		if(activescope instanceof PrintedScope && activescope!=null) ((PrintedScope)activescope).setActive(false);
		if(scope instanceof PrintedScope && scope!=null) ((PrintedScope)scope).setActive(true);
		this.activescope = scope;
		
	}
	
	
	public void addScope(Scope scope, int type){ 
		if(scope!=null) scopes.addScope(scope, type);
	}
	
	public Scope getScope(){
		return activescope;
	}	
	
	public AlternativeScopes getAlternativeScopes(){
		return scopes;
	}	
	
	
	public void setSymbolTable(SymbolTable stable){
		this.stable = stable;
	}
	
	public void removeFromSymbols(){
		if(stable!=null) stable.removeToken(this);
		setActiveScope(null);
		if(getParent() != null && getParent() instanceof ComplexGesture) ((ComplexGesture)getParent()).removeToken(this);
	}
	

// I assume that a 80% of the tokens points have to be within the shape
	public double isContainedIn(Shape shape){
		if(stroke == null) return 0;
		
		int counter_yes = 0;
		for(PenSample sample:stroke.getSamples())
			if(shape.contains(sample.getX(), sample.getY())) counter_yes++; 
//System.err.println(((float)counter_yes)/stroke.getSamples().size());	
		return ((float)counter_yes)/stroke.getSamples().size();
		
//		if(((float)counter_yes)/stroke.getSamples().size() >=.8) return true;
//		else return false;
	}
	
	public void setRemove() {
		removed = true;
		// TODO Auto-generated method stub
		
	}
	
	public void setFree(boolean free){
		//nothing happens.... only if Token instanceof GroupingGesture!
	}
	
	public boolean removed(){
		return removed;
	}
	
	public boolean isLeaf(){
		return true;
	}
	
	public boolean isAlone(){
		return isAlone;
	}
	
	public Collection<Token> getSubTokens(){
		return null;
	}

	public double getMinX(){
		return stroke.getMinX();
	}
	
	public double getMaxY(){
		return stroke.getMaxY();
	}
	
	public boolean detectScope(){ 
		if(types == null) return false;  
		
		Scope scope = getScope(types[0]);
		setActiveScope(scope);
		addScope(scope, types[0]);

		for(int i=1; i<types.length; ++i)
			addScope(getScope(types[i]), types[i]);
	
		if(activescope != null) return true;
		else return false;
	}
	
	Vector<ScorePart> getPoints(){
		return null;
	}
	
	// implemented only for simple Tokens that do not have any subtokens...
	public Scope getScope(int type){
		GeneralInkStroke stroke = getFirstStroke();
		SingleStrokeAnalysisResult result = stroke.getAnalysisResult();
		ScoreSkeleton scoreSkeleton = gcanvas.getPageSkeleton();
		
		switch(type){
			case Scope.CURVE_POINTS:
				Vector<ScorePart> points = getPoints();

				ScorePointer sp = new ScorePointer(points);
				sp.shape = stroke.getShape();
					
				return sp;
				
			case Scope.STAFF_CLOSEST:
				PolyStaff pstaff = result.getClosestPolyStaff();
				ScopeInstance scopeInstance = new ScopeInstance(result.getVerticalStartPointer(pstaff), result.getVerticalEndPointer(pstaff), result.getCloseVoiceLines());
				return new PrintedScope(gcanvas, pstaff, scopeInstance.getScope().get(scoreSkeleton), type);
				
			case Scope.STAFF_UNDER:
				VoiceLine voiceLine = result.getUnderVoiceLine();
				
				if(voiceLine != null){
					ScopeInstance scope = new ScopeInstance(voiceLine, 
						result.getVerticalStartPointer(voiceLine.getPolyStaff()), 
						result.getVerticalEndPointer(voiceLine.getPolyStaff()));
					
					return new PrintedScope(gcanvas, voiceLine.getPolyStaff(), scope.getScope().get(scoreSkeleton), type);
				}
				else return null;
				
			case Scope.STAFF_OVER:
				voiceLine = result.getOverVoiceLine();
				
				if(voiceLine != null){
					ScopeInstance scope = new ScopeInstance(voiceLine, 
						result.getVerticalStartPointer(voiceLine.getPolyStaff()), 
						result.getVerticalEndPointer(voiceLine.getPolyStaff()));
					
					return new PrintedScope(gcanvas, voiceLine.getPolyStaff(), scope.getScope().get(scoreSkeleton), type);
				}
				else return null;
				
			case Scope.MULTISTAFF_UNDER:
				pstaff = result.getUnderPolyStaff();
				if(pstaff!=null) {
					ScopeInstance scope = new ScopeInstance(result.getVerticalStartPointer(pstaff), result.getVerticalEndPointer(pstaff));
					return new PrintedScope(gcanvas, pstaff, scope.getScope().get(scoreSkeleton), type);
				}
				else return null;
				
			case Scope.MULTISTAFF_OVER:
				pstaff = result.getOverPolyStaff();
				if(pstaff!=null) {
					ScopeInstance scope = new ScopeInstance(result.getVerticalStartPointer(pstaff), result.getVerticalEndPointer(pstaff));
					return new PrintedScope(gcanvas, pstaff, scope.getScope().get(scoreSkeleton), type);
				}
				else return null;
				
			case Scope.INSIDE:
				pstaff = result.getClosestPolyStaff();
				PrintedScope scope = new PrintedScope(gcanvas, pstaff, stroke.getShape(), type);
				if(!scope.getSymbols().isEmpty()) return scope;	
				else return null;						
				//else return scope;	
			
			case Scope.NOTES_UNDER:
				voiceLine = result.getUnderVoiceLine();
				if(voiceLine!=null){				
					ScorePart[] notes = stroke.getNotes(voiceLine);
					if(notes == null || notes.length == 0) return null;
					
					ChordsScope scope1 = new ChordsScope(gcanvas, voiceLine.getPolyStaff(), notes, type);
					return scope1;
				}
				else return null;
			
			case Scope.NOTES_UP:
				voiceLine = result.getOverVoiceLine();
				if(voiceLine!=null){				
					ScorePart[] notes = stroke.getNotes(voiceLine);
					if(notes == null || notes.length == 0) return null;
					
					ChordsScope scope1 = new ChordsScope(gcanvas, voiceLine.getPolyStaff(), notes, type);
					return scope1;
				}
				else return null;
				
			case Scope.NOTES_CLOSEST:
				voiceLine = result.getClosestVoiceLine();
				if(voiceLine!=null){				
					ScorePart[] notes = stroke.getNotes(voiceLine);
					if(notes == null || notes.length == 0) return null;
					
					ChordsScope scope1 = new ChordsScope(gcanvas, voiceLine.getPolyStaff(), notes, type);
					return scope1;
				}
				else return null;
							
			case Scope.CENTER_UP:
				voiceLine = result.getOverVoiceLine();
				if(voiceLine!=null){				
					ScorePart[] notes = stroke.getNotes(voiceLine);
					if(notes == null || notes.length == 0) return null;
					
					ChordsScope scope1 = new ChordsScope(gcanvas, voiceLine.getPolyStaff(), notes, type);
					return scope1;
				}
				else return null;
				
			case Scope.CENTER_UNDER:
				voiceLine = result.getUnderVoiceLine();
				if(voiceLine!=null){				
					ScorePart[] notes = stroke.getNotes(voiceLine);
					if(notes == null || notes.length == 0) return null;
					
					ChordsScope scope1 = new ChordsScope(gcanvas, voiceLine.getPolyStaff(), notes, type);
					return scope1;
				}
				else return null;
				
			case Scope.CENTER_CLOSEST:
				voiceLine = result.getClosestVoiceLine();
				if(voiceLine!=null){				
					ScorePart[] notes = stroke.getNotes(voiceLine);
					if(notes == null || notes.length == 0) return null;
					
					ChordsScope scope1 = new ChordsScope(gcanvas, voiceLine.getPolyStaff(), notes, type);
					return scope1;
				}
				else return null;
				
			default:
				return null;
								
		}
	}


	public boolean closeTo(Token token, int maxDist) {
		if(maxDist < 0) return true;
		
		Rectangle2D bounds1 = token.stroke.getBounds();
		Rectangle2D bounds2 = stroke.getBounds();
		
		if(new Point2D.Double(bounds1.getX(), bounds1.getY()).distanceSq(
				new Point2D.Double(bounds2.getX(), bounds2.getY())) < Math.pow(maxDist, 2))
			return true;
		else return false;
	}

	public boolean intersects(Line2D line) {
		return getFirstStroke().intersects(line);
	}
	
	public Token getStartConnection(Collection<Token> tokens){
		for(Token token: tokens){
			if(token == this || gcanvas != token.gcanvas) continue;	
			if(stroke.startsFrom(token.stroke)) return token;	
		}
		
		for(Token token: tokens){
			if(token == this || gcanvas != token.gcanvas) continue;	
			if(stroke.startsAbout(token.stroke)) return token;	
		}
		
		return null;
	}

	public Token getEndConnection(Collection<Token> tokens, Token startToken){
		for(Token token: tokens){
			if(token == this || gcanvas != token.gcanvas) continue;
			
			if(stroke.endsTo(token.stroke) && token!=startToken) return token;	
		}
		
		for(Token token: tokens){
			if(token == this || gcanvas != token.gcanvas) continue;
			
			if(stroke.endsAbout(token.stroke) && token!=startToken) return token;	
		}
		
		
		return null;
	}


	public Token getIntersectedToken(Collection<Token> tokens) {
		return null;
	}


	public Token intersectsWith(Token token) {
		if(token.stroke == null) return token.intersectsWith(this); 
			
		if(stroke.inShape(token.stroke.getShape())) return this;
		else return null;
	}	

	public Token getProximityToken(Collection<Token> tokens) { 
		return null;
	}
	
	public double distanceFrom(Token token){
		if(intersectsWith(token)!=null) return 0;
		
		if(token.stroke == null)		
			return token.distanceFrom(this); 

		return (stroke.distanceFrom(token.stroke));
	}

	
	public String toString(){	
		
		return "Unrecognized stroke";
		/*
		String classname =  getClass().getName();
		//toString()
		return classname.substring(classname.lastIndexOf(".") + 1);
		//return  + "\n" + ((connections!=null)?connections.toString():"");
		 
		 */
	}

	public Token getContainerToken() {
		if(getParent() instanceof GroupingGesture) return getParent();
		else return this;
	}


	Vector<Token> dependantTokens;
	public void addDependantToken(Token token) {
		if(dependantTokens == null) dependantTokens = new Vector<Token>();
		dependantTokens.add(token);		

//		System.err.println(this + " ... " + token);
	}
	
	public void removeDependantGestures(){//System.err.println(" &&& " + this);
		if(dependantTokens == null) return;
	
		for(Token token:dependantTokens){
			if(token.getGesturePath(gcanvas) !=null) token.getGesturePath(gcanvas).delete();
		}
	}


	public boolean isRecognizable() {
		return true;
	}
	
	// public int getReturn
	GeneralizedGestureClass gclass = null;
//	boolean isCommand = false;
	public void setGestureClass(GeneralizedGestureClass gclass) {
		this.gclass = gclass;
//		isCommand = true;
	}
	
	
	public String getOperation(){
		if(gclass == null || gclass.getDefinition() == null) return null;
		return gclass.getDefinition().getOperation();
	}
	
	String parameter = null;
	public void setParameter(String name){
		parameter = name;
	}
	
	public String getParameter(){
		return parameter;
	}


	int recognizedType = -1;
	public void setRecognizedType(int type) {
		recognizedType = type;		
	}	
	
	public int getRecognizedType() {
		return recognizedType;		
	}	
	
	public boolean intersects(Rectangle2D pointBorders) {
		
		for(PenSample sample:stroke.getSamples()){
			if(pointBorders.contains(sample.getX(), sample.getY()))
					return true;
		}
		
		return false;
	}
	
	public int compareScorePositionWith(Token t){
		PolyStaff p1 = activescope.pstaff;
		PolyStaff p2 = t.activescope.pstaff;
		
		if(p2 == null) return -1;
		else if(p1 == null) return 1;
		else if(p1.getID() == p2.getID()) 
			return (stroke.getBounds().getMinX() <= t.stroke.getBounds().getMinX())?-1:1;
		else if(p1.getID() < p2.getID()) return -1;
		else return 1;	
	}
	
	///////////
	protected static  Object[] filterPoints(List<PenSample> list){
		
		PenSample currentSample = null;
		Vector<PenSample> newlist = new Vector<PenSample>();
		
		for(PenSample sample:list){
			if(currentSample == null || sample.getX() != currentSample.getX() || sample.getY() != currentSample.getY())
				newlist.addElement(sample);
		}
			
		if(newlist.size() <= GlobalParameters.MAX_OSC_SIZE) return newlist.toArray();
	
		double sampleStep = (double)newlist.size()/GlobalParameters.MAX_OSC_SIZE;
		
		Object[] fsamples = new Object[GlobalParameters.MAX_OSC_SIZE];
		for(int i=0; i<GlobalParameters.MAX_OSC_SIZE; ++i)
			fsamples[i] = newlist.elementAt((int)Math.round(sampleStep*i));
		
		return fsamples;
	}
	
	protected static int getAmplitude(double y, PolyStaff staff){
		return (int)Math.round(GlobalParameters.MAX_AMPLITUDE - (y - staff.getMinY())*(GlobalParameters.MAX_AMPLITUDE -
					GlobalParameters.MIN_AMPLITUDE)/staff.getHeight());
	}
	
	protected static int getTime(double x, PolyStaff staff){
		if(x < staff.getMinX()) x = staff.getMinX();
		else if(x > staff.getMaxX()) x = staff.getMaxX();
			
		return (staff.getID() - 1)*GlobalParameters.STAFF_LENGTH + (int)Math.round((x - staff.getMinX())*GlobalParameters.STAFF_LENGTH/staff.getWidth());
	}
	
	
	public String getCurvePoints(){
		SingleStrokeAnalysisResult result = stroke.getAnalysisResult();
		PolyStaff staff = result.getClosestPolyStaff();
			
		Object[] samples = filterPoints(stroke.getSamples());
		
		String s = "";
		s+="( ";
		
		for(int i = 0; i<samples.length; ++i){
			s += (getTime(((PenSample)samples[i]).getX(), staff) + " ");
		}
		s+=") ( ";
		for(int i = 0; i<samples.length; ++i){
			s += (getAmplitude(((PenSample)samples[i]).getY(), staff) + " ");
		}
		s+=")";
		
		return "(" + s + ")";
	}
	
	
	public int getTimePosition(){
		SingleStrokeAnalysisResult result = stroke.getAnalysisResult();
		PolyStaff staff = result.getClosestPolyStaff();
		
		double x_min = 10000000;
		for(PenSample sample:stroke.getSamples()){
			if(sample.getX() < x_min) x_min = sample.getX();
		}
		
		return getTime(x_min, staff);
	}
	
	public String getXRange(){
		SingleStrokeAnalysisResult result = stroke.getAnalysisResult();
		PolyStaff staff = result.getClosestPolyStaff();
		
		double x_min = 10000000, x_max = 0;
		for(PenSample sample:stroke.getSamples()){
			if(sample.getX() < x_min) x_min = sample.getX();
			if(sample.getX() > x_max) x_max = sample.getX();
		}
		
		return "(" + getTime(x_min, staff) + " " + getTime(x_max, staff) + ")";
	}
	
	public String getStartPoint(){
		SingleStrokeAnalysisResult result = stroke.getAnalysisResult();
		PolyStaff staff = result.getClosestPolyStaff();
		
		PenSample sample = stroke.getSamples().get(0);
		
		return "(" + getTime(sample.getX(), staff) + " " + getAmplitude(sample.getY(), staff) + ")";
	}
	
	public String getCentralPoint(){
		SingleStrokeAnalysisResult result = stroke.getAnalysisResult();
		PolyStaff staff = result.getClosestPolyStaff();
		
		Rectangle2D bounds = stroke.getBounds();
		
		return "(" + getTime(bounds.getCenterX(), staff) + " " + getAmplitude(bounds.getCenterY(), staff) + ")";
	}
	
	
	/////////
	
	public OSCBundle getOSCBundle(OutputWindow outwin, SortedMap<String, Operation> operations){
		OSCBundle bundle = new OSCBundle(System.currentTimeMillis());;
		String name;
		Vector<Object> param = new Vector<Object>();
		
		String operName = getOperation() ;
		Operation operation = (operName == null) ? null:operations.get(operName);
		//Horizontal token = (Horizontal)this;
			
		if(operation == null){
			param.add("multi-seq");
				
			param.add("" + getCurvePoints());
			name = "/generic";
		}
		else{
			name = "/" + operation.getName();
			param.add("multi-seq");
				
			for(OperationArgument arg:operation.getArguments()){
				switch(arg.getType()){
					case Operation.POINTS: 
						if(this instanceof GroupingGesture && !connections.isEmpty()){
							Connection connection = connections.first();
							Token tok = connection.connected;
							param.add("" + tok.getCurvePoints());								
						}
						else param.add("" + getCurvePoints());
						break;
					case Operation.START:
						if(this instanceof GroupingGesture && !connections.isEmpty()){
							Connection connection = connections.first();
							Token tok = connection.connected;
							param.add("" + tok.getStartPoint());								
						}
						else param.add("" + getStartPoint());
						break;
					case Operation.CENTER: 
						if(this instanceof GroupingGesture && !connections.isEmpty()){
							Connection connection = connections.first();
							Token tok = connection.connected;
							param.add("" + tok.getCentralPoint());								
						}
						else param.add("" + getCentralPoint());
						break;
					case Operation.XRANGE:
						if(this instanceof GroupingGesture && !connections.isEmpty()){
							Connection connection = connections.first();
							Token tok = connection.connected;
							param.add("" + tok.getXRange());								
						}
						else param.add("" + getXRange());
						break;
					case Operation.PARAMETER: //accommodates multiple parameters
						for(Connection connection:connections){
							Token tok = connection.connected;
							if(tok instanceof GroupingGesture){
								GesturePath gpath = tok.gestures.elements().nextElement();						
								GeneralizedGestureClass gclass = gpath.getGestureClass();
								if(gclass != null) param.add("" + gclass.getName());
							}
						}
						break;	
				}
					
			}
		}

		
		if(outwin!=null) outwin.printMessage(name, param);
		OSCMessage mes = new OSCMessage(name, param.toArray());
		bundle.addPacket(mes);
		
		return bundle;
		
	}
	
}
