package inria.paperoles.io;

import inria.paperoles.scoremodel.Chord;
import inria.paperoles.scoremodel.GraphVoiceLine;
import inria.paperoles.scoremodel.InteractionBox;
import inria.paperoles.scoremodel.Measure;
import inria.paperoles.scoremodel.MultiplePageSkeleton;
import inria.paperoles.scoremodel.Note;
import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.scoremodel.ScoreSkeleton;
import inria.paperoles.scoremodel.Segment;
import inria.paperoles.scoremodel.Voice;
import inria.paperoles.scoremodel.VoiceLine;

import java.awt.Rectangle;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;

public class ScoreDescriptionParser {
		
	ScoreSkeleton skeleton;	
	
	File notesFile;
	int off_x, off_y;
	
	
	public ScoreDescriptionParser(File file){
		notesFile = file;			
	}
	
	public MultiplePageSkeleton build(int off_x, int off_y) throws NumberFormatException, IOException {
		MultiplePageSkeleton multiskeleton = new MultiplePageSkeleton();
		Voice.initialize();
		Measure.resetCounter();
		
		BufferedReader input = new BufferedReader(new FileReader(notesFile));
		String source = getMusicScore(input);
		
		this.off_x = off_x;
		this.off_y = off_y;
		
		InputEntry notationBox = null, annotationBox = null;
		
		notationBox = getBox(input);
		if(notationBox != null) annotationBox = getBox(input);
		if(notationBox != null && annotationBox !=null) entry = null;
		
	//	int pageNum = getPageNum(input); 
		for(ScoreSkeleton pageSkeleton = getNextPage(input, multiskeleton); pageSkeleton != null; pageSkeleton = getNextPage(input, multiskeleton)){
			multiskeleton.addPageSkeleton(pageSkeleton);
	
			if(annotationBox!=null && notationBox!=null){ 
				pageSkeleton.addAnnotationBox(new InteractionBox(annotationBox.getX(), annotationBox.getY(), 
						annotationBox.getWidth(), annotationBox.getHeight()));
				
				pageSkeleton.addNotationBox(new InteractionBox(notationBox.getX(), notationBox.getY(), 
						notationBox.getWidth(), notationBox.getHeight()));
			}
		}
		
		multiskeleton.setSource(source);
		return multiskeleton;
	}
	
	/// Normally, i should first call buildSkeleton and then readNotes.
	// in order to build the model correctly...
	int polyStaffNum = 1;
	private ScoreSkeleton getNextPage(BufferedReader input, 
			MultiplePageSkeleton multiskeleton) throws NumberFormatException, IOException {

		if(entry == null){ 
			String line = input.readLine();
		
			if(line == null) return null;
			entry = new InputEntry(line, off_x, off_y);
		}
			
		if(!entry.isPage()) return null;	
		
		skeleton = new ScoreSkeleton(multiskeleton, entry.isGraphPage());
		entry = null;
		
		for(PolyStaff polyStaff = getNextPolyStaff(input, skeleton, polyStaffNum); polyStaff != null; polyStaff = getNextPolyStaff(input, skeleton, ++polyStaffNum))
			skeleton.addDocPart(polyStaff);
		
		return skeleton;
	}

	InputEntry entry;
	private PolyStaff getNextPolyStaff(BufferedReader input, ScoreSkeleton skeleton, int num) throws IOException {
		if(entry == null){
			String line = input.readLine();
			
			if(line == null) return null;
			entry = new InputEntry(line, off_x, off_y); 
		}
		
		if(!entry.isMultiStaff()) return null;
						
		PolyStaff polyStaff = new PolyStaff(skeleton, num, entry.getX(), entry.getY(), 
				entry.getWidth(), entry.getHeight());
	
		entry = null;
		
		int i = 1;
		for(VoiceLine staff = getNextStaff(input, polyStaff, i); staff != null; staff = getNextStaff(input, polyStaff, ++i)){
			//polyStaff.addVoiceLine(staff);
			skeleton.addDocPart(staff);
		}
		
		return polyStaff;
	}


	private VoiceLine getNextStaff(BufferedReader input, PolyStaff polyStaff, int num) throws IOException {
		if(entry == null){
			String line = input.readLine();
			
			if(line == null) return null;
			entry = new InputEntry(line, off_x, off_y);
		}
		
		if(!entry.isStaff() && !entry.isGraphstaff()) return null;
				
		VoiceLine staff;
		
		if(entry.isStaff())
			staff = new VoiceLine(polyStaff, num, entry.getX(), entry.getY(), entry.getWidth(), entry.getHeight());
		else staff = new GraphVoiceLine(polyStaff, num, entry.getX(), entry.getY(), entry.getWidth(), entry.getHeight());		
		
		entry = null;
		
		int i = 0;
		for(Segment segment = getNextSegment(input, staff, i); segment != null; segment = getNextSegment(input, staff, ++i)){
			//staff.addSegment(measure);
		}
		
		return staff;
	}

	
	private Segment getNextSegment(BufferedReader input, VoiceLine staff, int num) throws IOException {		
		if(entry == null){
			String line = input.readLine();
			
			if(line == null) return null;
			entry = new InputEntry(line, off_x, off_y);
		}
		
		if(!entry.isMeasure()) return null;
			
		Measure measure = staff.getPolyStaff().getMeasureAt(num);
		if(measure == null) measure = new Measure();
		Segment segment = new Segment();
		
		int y1 = (int)Math.min(staff.getMinY(), entry.getY());
		int y2 = (int)Math.max(staff.getMaxY(), entry.getY() + entry.getHeight());

		if(measure.getPolyStaff() == null){
			measure.initialize(staff.getPolyStaff(), entry.getX(), entry.getX() + entry.getWidth() - 1);
			skeleton.addDocPart(measure);
		}
		segment.initialize(measure, staff, entry.getX(), y1, entry.getWidth(), y2-y1);
		skeleton.addDocPart(segment);

		entry = null;
		
		for(Chord chord = getNextChord(input, segment); chord != null; chord = getNextChord(input, segment))
			skeleton.addDocPart(chord);
		
		return segment;
	}
	
	
	private Chord getNextChord(BufferedReader input, Segment segment) throws IOException {
		if(entry == null){
			String line = input.readLine();
			
			if(line == null) return null;
			entry = new InputEntry(line, off_x, off_y);
		}
		
		if(entry.isGroup()){
			entry = null;
			return getNextChord(input, segment);
		}
		
		if(!entry.isChord()) return null;	
		
		Chord chord = new Chord(segment, entry.getX(), entry.getY(), 
				entry.getWidth(), entry.getHeight());
			
		entry = null;

		for(Note note = getNextNote(input,chord); note != null; note = getNextNote(input, chord)){ 
			skeleton.addSymbol(note);		
		}
		
		return chord;
	}
	
	
	private Note getNextNote(BufferedReader input, Chord chord) throws IOException {
		if(entry == null){
			String line = input.readLine();
			
			if(line == null) return null;
			entry = new InputEntry(line, off_x, off_y);
		}
		
		if(!entry.isNote()) return null;		
		
		Note note = new Note(chord, entry.getX(), entry.getY(), 
				entry.getWidth(), entry.getHeight());
			
		entry = null;
		
		return note;
	}

	private String getMusicScore(BufferedReader input) throws IOException {
		String line = input.readLine();
		String[] par = line.substring(0, line.length() - 1).split(" ");
		if(par.length < 1) throw new IOException("wrong number of arguments");
		//if(!par[0].equals("score")) throw new IOException("expected 'score' as first argument");
		
		return par[0];
	}
	
	private InputEntry getBox(BufferedReader input) throws IOException{
		String line = input.readLine();
		
		if(line == null) return null;
		entry = new InputEntry(line, off_x, off_y);
		
		if(entry.isNotation() || entry.isAnnotation()) return entry;
		else return null;
	}
	
	private String[] getParameters(String line){
		return line.substring(1, line.length() - 1).split(" ");
	}
	
	class InputEntry{
		static final short ERROR = -1;
		static final short PAGE = 0;
		static final short MULTISTAFF = 1;
		static final short STAFF = 2;
		static final short MEASURE = 3;
		static final short GROUP = 4;
		static final short CHORD = 5;
		static final short NOTE = 6;
		static final short GRAPHSTAFF = 9;
		
		static final short NOTATION = 7;
		static final short ANNOTATION = 8;
		
		final static double scale = 1;//.48 
		
		int type = -1;
		
		int x, y, w, h;
		int pageNum = -1;
		
		InputEntry(String line, int off_x, int off_y){
			String []par = getParameters(line);
	
			if(par.length >= 1 && par[0].equalsIgnoreCase("page")){	
				type = PAGE;
				pageNum = (new Integer(par[1])).intValue();
				return;
			}
			else if(par.length != 5){
				type = ERROR; 
				return;
			}
			
			x = (int)(scale*((new Integer(par[1])).intValue() + off_x));
			y = (int)(scale*((new Integer(par[2])).intValue()  + off_y));
			w = (int)(scale*(new Integer(par[3])).intValue());
			h = (int)(scale*(new Integer(par[4])).intValue());
			
			if(par[0].equalsIgnoreCase("multistaff")) type = MULTISTAFF;
			else if(par[0].equalsIgnoreCase("staff")) type = STAFF;
			else if(par[0].equalsIgnoreCase("graphstaff")) type = GRAPHSTAFF;
			else if(par[0].equalsIgnoreCase("measure")) type = MEASURE;
			else if(par[0].equalsIgnoreCase("group")) type = GROUP;
			else if(par[0].equalsIgnoreCase("chord")) type = CHORD;
			else if(par[0].equalsIgnoreCase("note")) type = NOTE;
			else if(par[0].equalsIgnoreCase("notation")) type = NOTATION;
			else if(par[0].equalsIgnoreCase("annotation")) type = ANNOTATION;
			else type = ERROR;
			
		//	if(type == NOTE) y+=10;		
		}
		

		int getX(){
			return x;
		}
		
		int getY(){
			return y;
		}
		
		int getWidth(){
			return w;
		}
		
		int getHeight(){
			return h;
		}
			
		boolean isStaff(){
			return type == STAFF;
		}

		public boolean isGraphstaff() {
			return type == GRAPHSTAFF;
		}
		
		boolean isMultiStaff(){
			return type == MULTISTAFF;
		}
		
		boolean isNote(){
			return type == NOTE;
		}
		
		boolean isMeasure(){
			return type == MEASURE;
		}
		
		boolean isPage(){
			return type == PAGE;
		}
		
		boolean isGraphPage(){
			return pageNum == 0;
		}
		
		int getPageNum(){
			return pageNum;
		}
		
		boolean isGroup(){
			return type == GROUP;
		}

		boolean isChord(){
			return type == CHORD;
		}
		
		boolean isNotation(){
			return type == NOTATION;
		}

		boolean isAnnotation(){
			return type == ANNOTATION;
		}
		
		public String toString(){
			return ""+ type;
		}
	}
	
}
