
package inria.paperoles.openmusic;

import inria.paperoles.docmodel.Symbol;
import inria.paperoles.grammar.GroupingGesture;
import inria.paperoles.grammar.PrintedScope;
import inria.paperoles.grammar.Token;
import inria.paperoles.operations.MatchedOperation;
import inria.paperoles.operations.Operation;
import inria.paperoles.operations.OperationArgument;
import inria.paperoles.scoremodel.Note;
import inria.paperoles.ui.GesturePath;
import inria.paperoles.ui.OutputWindow;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.SortedMap;
import java.util.Vector;

import papertoolkit.pen.PenSample;

import de.sciss.net.OSCBundle;
import de.sciss.net.OSCChannel;
import de.sciss.net.OSCClient;
import de.sciss.net.OSCListener;
import de.sciss.net.OSCMessage;
import de.sciss.net.OSCServer;

public class OpenMusicClient {
	final static String PROTOCOL = OSCChannel.UDP;
	final static String HOSTNAME = "localhost";
//	final static int PORT = 1968;
	public static int PORT = 3333;	
	
	OSCClient client = null;
	OutputWindow outwin = null;
	
	 SortedMap<String, Operation> operations;
	
	
	public OpenMusicClient(OutputWindow outwin, SortedMap<String, Operation> operations) throws IOException{		
		this(operations);
	
		this.outwin = outwin;
	}
	
	public OpenMusicClient(SortedMap<String, Operation> operations){		
			this.operations = operations;
		
			try {
				client = OSCClient.newUsing(PROTOCOL);
				
				client.setTarget(new InetSocketAddress(HOSTNAME, PORT));
				client.start();	
				
			} catch (IOException e) {
				// TODO Auto-generated catch block
			//	e.printStackTrace();
			}
	}
	
	
	public void submitRecognizedOperations(OutputWindow win, Collection<Token> tokens) throws IOException{		
		OSCBundle bundle;
		
		///  send recognized operations
		for(Token token:tokens){

			bundle = token.getOSCBundle(win, operations);
			
			client.send(bundle);
		}				
	}
	
	public void kill(){
		if(client!=null) client.dispose();
	}

	public void sendCommand(GesturePath gesture) {
		try {
			OSCBundle bundle = gesture.getToken().getOSCBundle(outwin, operations);
			if(client!=null) client.send(bundle);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	private void test() throws IOException{
		OSCBundle bundle = new OSCBundle(System.currentTimeMillis());
		
		/////////////////////////////////////////////////////////////////
		////////////////////
		String name = "/skata";		
		
		
		OSCMessage mes = new OSCMessage(name, 
				new Object[]{"( (1322 1323 1314 1315 1326 1327 1338 1348 1359 1379 1322 1323 1314 1315 1326 1327 1338 1348 1359 1379 1322 1323 1314 1315 1326 1327 1338 1348 1359 1379 1322 1323 1314 1315 1326 1327 1338 1348 1359 1379 ) " +
						"( 164 163 164 166 205 276 307 338 408 458 666 999 1314 1315 1326 1327 1338 1348 1359 1379 1322 1323 1314 1315 1326 1327 1338 1348 1359 1379 1322 1323 1314 1315 1326 1327 1338 1348 1359 1379) )"
			});
		
				
	//	OSCMessage mes = new OSCMessage(name, 
	//			new Object[]{"((20 30 40) (20 100 500))"});
		bundle.addPacket(mes);
		
		client.send(bundle);
		
		client.stop();
		client.dispose();
	}
	
	public static void main(String[] args){
		/*
		try {
			(new OpenMusicClient()).test();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.err.println("...done");
		*/
	}
}
