package inria.paperoles.scoremodel;

import java.util.Collection;
import java.util.Vector;

import inria.paperoles.docmodel.DocPart;

public class PolyStaff extends ScorePart{
	int id;
	Vector<Measure> measures;
	Vector<VoiceLine> voiceLines;
	ScoreSkeleton skeleton;
	
	public PolyStaff(ScoreSkeleton skeleton, int id, int x, int y, int w, int h){
		super(x, y, w, h);
		
		this.id = id;
		
		this.skeleton = skeleton;
		measures = new Vector<Measure>();
		voiceLines = new Vector<VoiceLine>();
	}
	
	public int getID(){
		return id;
	}
	
	public ScoreSkeleton getSkeleton(){
		return skeleton;
	}
	
	public int compareWith(PolyStaff polystaff){
		/*
		int i = skeleton.compareWith(polystaff.skeleton);
		if(i < 0) return -1;
		else if(i > 0) return 1;
		*/
		
		if(id < polystaff.id) return -1;
		else if(id == polystaff.id) return 0;
		else return 1;
	}
	
	public ScoreVerticalPointer getStartPointer(){
		return new  ScoreVerticalPointer(this, getX());
	}
	
	public ScoreVerticalPointer getEndPointer(){
		return new  ScoreVerticalPointer(this, getX() + getWidth());
	}
	
	public String toString(){
		return "poly : " + id;
	}
	
	
	public PolyStaff getPreviousInPage(){
		Vector<PolyStaff> staffs = skeleton.polyStaffs;
		
		if(this == staffs.firstElement()) return null;
		else return skeleton.polyStaffs.get(staffs.indexOf(this) - 1);
	}
	
	public PolyStaff getNextInPage(){
		Vector<PolyStaff> staffs = skeleton.polyStaffs;
	
		if(this == staffs.lastElement()) return null;
		else return staffs.get(staffs.indexOf(this) + 1);
	}
	
	
	public PolyStaff getPrevious(){
		Vector<PolyStaff> staffs = skeleton.polyStaffs;
		
		if(this == staffs.firstElement()){
			ScoreSkeleton previousSkeleton = skeleton.getPrevious();
			
			if(skeleton == null) return null;
			else return previousSkeleton.getLastPolyStaff();
		}
		return skeleton.polyStaffs.get(staffs.indexOf(this) - 1);
	}
	
	public PolyStaff getNext(){
		Vector<PolyStaff> staffs = skeleton.polyStaffs;
	
		if(this == staffs.lastElement()){
			ScoreSkeleton nextSkeleton = skeleton.getNext();
			
			if(nextSkeleton == null) return null;
			else return nextSkeleton.getFirstPolyStaff();
		}
		else return staffs.get(staffs.indexOf(this) + 1);
	}
	
	public Measure getFirstMeasure(){
		return measures.firstElement();
	}
	
	public Measure getLastMeasure(){
		return measures.lastElement();
	}
		
	public Measure getMeasureAtX(double x){
		for(Measure mes:measures){
			if(mes.getMinX() <= x && mes.getMaxX()>=x)
				return mes;
		}
		return null;
	}
	
	public void addMeasure(Measure measure){
		measures.add(measure);
	}

	public void addVoiceLine(VoiceLine voiceLine){
		voiceLines.add(voiceLine);
	}
	
	public Vector<Measure> getMeasures(){
		return measures;
	}
	
	public Measure getMeasureAt(int i){
		if(i >= measures.size()) return null;
		return measures.elementAt(i);
	}
	
	public Vector<VoiceLine> getVoiceLines(){
		return voiceLines;
	}	
}
