package inria.paperoles.scoremodel;

import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.Area;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import inria.paperoles.docmodel.DocSkeleton;
import inria.paperoles.docmodel.Symbol;
import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.SingleInkStroke;
import inria.paperoles.gestures.SingleStrokeAnalysisResult;
import inria.paperoles.scoping.ExtendedShape;
import inria.paperoles.ui.GesturesCanvas;

public class ScoreSkeleton extends DocSkeleton {

	Vector<PolyStaff> polyStaffs;
	MultiplePageSkeleton multiskeleton;

	GesturesCanvas canvas;
	boolean isGraphPage = false;
	
	public ScoreSkeleton(MultiplePageSkeleton multiskeleton){
		this(multiskeleton, false);
	}

	public ScoreSkeleton(MultiplePageSkeleton multiskeleton, boolean isGraphPage) {
		super();

		polyStaffs = new Vector<PolyStaff>();
		this.multiskeleton = multiskeleton;
		this.isGraphPage = isGraphPage;
	}

	public void addSymbol(Note note){
		super.addSymbol(note);
	}	

	public boolean isGraphPage(){
		return isGraphPage;
	}
	
	
	public void addDocPart(ScorePart part){	
		super.addDocPart(part);
	
		if(part instanceof PolyStaff) polyStaffs.add((PolyStaff)part);
	}
	
	
	public PolyStaff getFirstPolyStaff(){
		return polyStaffs.firstElement();
	}
	
	public PolyStaff getLastPolyStaff(){
		return polyStaffs.lastElement();
	}
	
	public ScoreSkeleton getPrevious(){
		int i = getIndex() - 1;
		if(i < 0) return null;
		else return multiskeleton.get(i);
	}
	
	public ScoreSkeleton getNext(){
		int i = getIndex() + 1;
		if(i == multiskeleton.size()) return null;
		else return multiskeleton.get(i);
	}
	
	public int getIndex(){
		return multiskeleton.indexOf(this);
	}
	
	public int compareWith(ScoreSkeleton skeleton){
		int id1 = getIndex();
		int id2 = skeleton.getIndex();
		
		if(id1 < id2) return -1;
		else if(id1 == id2) return 0;
		else return 1;
	}
	
	public Vector<Segment> createScoreSegments(){
		Vector<Segment> segments = new Vector<Segment> ();
	
		for(PolyStaff polyStaff:polyStaffs)
			for(VoiceLine voiceLine:polyStaff.voiceLines)
				for(Measure measure:polyStaff.measures)
					segments.add(new Segment(measure, voiceLine));	

		return segments;
	}
	

	public Vector<PolyStaff> getPolyStaffs(){
		return polyStaffs;
	}
	
	
	public String toString(){
		return "skeleton " + multiskeleton.indexOf(this);
	}
	
	
	
	private Vector<Segment> findIntersectedSegments(Shape shape){
		Vector<Segment> segments = new Vector<Segment>();
		
		for(PolyStaff polyStaff:polyStaffs)
			for(VoiceLine voiceLine:polyStaff.voiceLines)
				for(Segment segment:voiceLine.getSegments())
					if(shape.intersects(segment.getBounds())) segments.add(segment);
		
		return segments;
	}
	
	private VoiceLine findClosestVoiceLine(Line2D line){
		double mindist = 50000, dist;
		VoiceLine closestStaff = null;
		
		for(PolyStaff polyStaff:polyStaffs)
			for(VoiceLine voiceLine:polyStaff.voiceLines)
				if((dist = line.ptLineDist(voiceLine.getCenter())) < mindist){
					mindist = dist;
					closestStaff = voiceLine;
				}
		
		return closestStaff;
	}

	public ScoreVerticalPointer getStartPointer(){
		return new  ScoreVerticalPointer(polyStaffs.firstElement(), getX());
	}
	
	public ScoreVerticalPointer getEndPointer(){
		return new  ScoreVerticalPointer(polyStaffs.firstElement(), getX() + getWidth());
	}
	
	public Collection<Symbol> select(ExtendedShape shape){
		return super.select(shape.get(this));
	}

	public void setCanvas(GesturesCanvas gesturesCanvas) {
		canvas = gesturesCanvas;
	}	
	
	public GesturesCanvas getCanvas(){
		return canvas;
	}
	
	InteractionBox annotationBox;
	public void addAnnotationBox(InteractionBox interactionBox) {
		addDocPart(interactionBox);
		annotationBox = interactionBox;
	}

	InteractionBox notationBox;
	public void addNotationBox(InteractionBox interactionBox) {
		addDocPart(interactionBox);
		notationBox = interactionBox;
	}
	
	public boolean inAnnotationBox(SingleInkStroke stroke){

		if(annotationBox != null && stroke.isInside(annotationBox.getBounds())) return true;
		else return false;
	}
	
	public boolean inNotationBox(SingleInkStroke stroke){
		if(notationBox != null && stroke.isInside(notationBox.getBounds())) return true;
		else return false;
	}

	public void clean() {
		for(Symbol symbol:super.select(null)){
			if(symbol instanceof WrittenSymbol){
				((WrittenSymbol)symbol).removeFromModel();
				removeSymbol(symbol);
			}
		}
		
	}
}
