package inria.paperoles.scoremodel;

import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.SingleInkStroke;
import inria.paperoles.ui.GraphPaper;

import java.awt.Color;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.Vector;

import papertoolkit.pen.PenSample;

import edu.umd.cs.piccolo.nodes.PPath;

public class WaveLine extends Note implements WrittenSymbol{
	
	SingleInkStroke stroke;
	
	final static Color STROKE_COLOR = new Color(30, 30, 30);
	final static Color FOCUS_STROKE_COLOR = new Color(220, 150, 100);
	
	// this is the maximum number of points to be sent through OSC
	final static int MAX_OSC_SIZE = 40;
	
	public WaveLine(SingleInkStroke stroke){
		super();
		
		this.stroke = stroke;
		setStrokePaint(STROKE_COLOR);
		
		setPathTo(stroke.getShape());
		
	}
	
	
	public WaveLine(Chord chord, SingleInkStroke stroke) {
		super();
		
		this.stroke = stroke;
		setStrokePaint(STROKE_COLOR);
		
		setPathTo(stroke.getShape());
		
		setChord(chord);
		
		// TODO:
		// ok, I may also add here the points of the line as symbols
		// so that i can allow for interaction with portions 
		// of a wave line (e.g., a selection of points)
	}
		

	public Point2D getCenter(){
		return new Point2D.Double(getBounds().getCenterX(), getBounds().getCenterY());
	}
		
	public void setSkeleton(Line2D skeleton){
		addChild(new PPath(skeleton));
	}
		
	public Object clone(){
		WaveLine  wline = new WaveLine(stroke);
		return wline;
	}
		
//		private boolean selected = false;
	public void setFocus(boolean focus){
		if(focus) setStrokePaint(FOCUS_STROKE_COLOR);
		else setStrokePaint(STROKE_COLOR);
	}

	public void removeFromModel() {
		chord.segment.removeChord(chord);
	}
	
	public String toString(){
		return "Curve";
	}


	// not currently used....
	private void printToFile(String s){
		String fname = "curveFile.txt";
		
		try {
			BufferedWriter out = new BufferedWriter(new FileWriter(fname, false));
			
			out.write(s);;
			out.flush();
			out.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			
		}
	}
	
	
	public String stringRepresentation() {
		return getCurveRepresentation(stroke);
	}
	
	public static String getCurveRepresentation(GeneralInkStroke stroke){
		Object[] samples = filterPoints(stroke.getSamples().toArray());
		
		String s = "";
		
		double x_cent, y_cent;
		s+="( ";
		
		for(int i = 0; i<samples.length; ++i){
			x_cent = 100*(((PenSample)samples[i]).getX() - GraphPaper.off_x)/GraphPaper.CENTIMETER_STEP;
			s+= (int)Math.round(x_cent) + " ";
		}
		s+=") ( ";
		for(int i = 0; i<samples.length; ++i){
			y_cent = 17/*GraphPaper.getHeightInCent()*/ - (((PenSample)samples[i]).getY() - GraphPaper.off_y)/GraphPaper.CENTIMETER_STEP;
			s+= (int)Math.round(100*y_cent) + " ";
		}
		s+=")";
		
		/*
		// TODO needs scaling to centimeters...
		for(PenSample sample:stroke.getSamples())
			s+= " (" +(int)sample.getX() + ", " + (int)sample.getY() + ")";
		*/
		
		return "(" + s + ")";
	}
	
	public static String getCurveRepresentation(Point2D pref, GeneralInkStroke stroke){
		Object[] samples = filterPoints(stroke.getSamples().toArray());
		
		String s = "";
		
		double x_, y_;
		s+="( ";
		
		for(int i = 0; i<samples.length; ++i){
			x_ = (((PenSample)samples[i]).getX() - pref.getX());
			s+= (int)Math.round(x_) + " ";
		}
		s+=") ( ";
		for(int i = 0; i<samples.length; ++i){
			y_ = pref.getY() - ((PenSample)samples[i]).getY();
			s+= (int)Math.round(y_) + " ";
		}
		s+=")";
		
		return "(" + s + ")";
	}
	
	public static Object getCurveRepresentation(Measure containingMeasure,
			GeneralInkStroke stroke) {
		double x0 = containingMeasure.getX();
		double y0 = containingMeasure.getPolyStaff().getBounds().getCenterY();
		double length = containingMeasure.getWidth();
		
		Object[] samples = filterPoints(stroke.getSamples().toArray());
		
		String s = "";
		
		double x_, y_;
		s+="( ";
		
		for(int i = 0; i<samples.length; ++i){
			x_ = (1000*(((PenSample)samples[i]).getX() - x0))/length;
			s+= (int)Math.round(x_) + " ";
		}
		s+=") ( ";
		for(int i = 0; i<samples.length; ++i){
			y_ = y0 - ((PenSample)samples[i]).getY();
			s+= (int)Math.round(y_) + " ";
		}
		s+=")";
		
		return "(" + s + ")";
	}
	
	
	private static  Object[] filterPoints(Object[] samples){
		if(samples.length <= MAX_OSC_SIZE) return samples;
	
		double sampleStep = (double)samples.length/MAX_OSC_SIZE;
		
		Object[] fsamples = new Object[MAX_OSC_SIZE];
		for(int i=0; i<MAX_OSC_SIZE; ++i)
			fsamples[i] = samples[(int)Math.round(sampleStep*i)];
		
		return fsamples;
	}



	
}
