package inria.paperoles.ui;

import inria.paperoles.data.DataDispatcher;
import inria.paperoles.data.DataHistory;
import inria.paperoles.data.PenDataEntry;
import inria.paperoles.gestures.InkEventHandler;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;

import papertoolkit.pen.synch.BatchedDataDispatcher;

public class DataListWindow extends JDialog{
	
	String remoteSource;
	String localSource;
	
	DataHistory history;
	DataList fileList;
	FileWindowPanel panel;
	
	boolean scoreExists = false;
	
	DataDispatcher dispatcher = null;
	GestureBrowser parent;
	
	public DataListWindow(GestureBrowser parent, int x, int y){
		super(parent, "Available Pen Data");
		setLayout(new BorderLayout());
		
		setBounds(x - 100, y - 150, 200, 200);
		this.parent = parent;
		
		remoteSource = null;//GlobalParameters.DATA_URL;
		localSource = readSourceFormConfig(GlobalParameters.CONFIG_FILE);
		//localSource = GlobalParameters.LOCAL_DATA_URL;
		
		history = DataHistory.loadHistory();
		if(history == null) history = new DataHistory();

	//	downloadNewData();
		fetchNewData();
		history.save();
		
	    fileList = new DataList(this);
	    for(PenDataEntry entry:history.values())
	    	fileList.addEntry(entry, false);
	    
		JScrollPane scrollPane = new JScrollPane(fileList, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	//	scrollPane.setPreferredSize(new Dimension(200, 150));	
		scrollPane.setAlignmentX(LEFT_ALIGNMENT);

	    add(scrollPane, BorderLayout.CENTER);
	    
	    panel = new FileWindowPanel(this);
	    panel.setMinimumSize(new Dimension(200, 30));
	    add(panel, BorderLayout.SOUTH);
	    
		setVisible(true);
		
		dispatcher = new DataDispatcher();
	}
	
	private String readSourceFormConfig(String configFile) {
	
		try {
			BufferedReader reader = new BufferedReader(new FileReader(configFile));
			
			return reader.readLine() + "/";
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			return "";//GlobalParameters.LOCAL_DATA_URL;
		}
		
	}

	public void update(){
		fileList.assignTo(((GestureBrowser)getParent()).getSource());
		scoreExists = true;
	}
	
	
	public void dispatchData(String xmlDataFile){		
		((InkEventHandler)parent.getEventHandler()).reset();
		dispatcher.process(new File(xmlDataFile), parent.getEventHandler());
		
		parent.refresh();
	}
	
	public FileWindowPanel getButtonPanel(){
		return panel;
	}
	
	public JLabel getShowButton(){
		return panel.showlabel;
	}
	
	public JLabel getAssignButton(){
		return panel.assignlabel;
	}
	
	public Component getUnAssignButton() {
		// TODO Auto-generated method stub
		return panel.unassignlabel;
	}
	
	public void setDataSource(String remoteSource){
		this.remoteSource = remoteSource;
	}
	
	public String getDataSource(){
		return remoteSource;
	}
	
	public void loadDataHistory(){
		
	}
	
	public void saveDataHistory(){
		
	}
	
	public void refreshData(){
		if(/*!downloadNewData() &&*/ !fetchNewData()) return;
		history.save();

		showHistory();
	}
	
	private void showHistory(){	
		fileList.clear();
	    for(PenDataEntry entry:history.values())
	    	fileList.addEntry(entry, 
	    			entry.getScoreFile()!=null &&
	    			entry.getScoreFile().equals(((GestureBrowser)getParent()).getSource()));
	}
	
	public boolean downloadNewData() {	
		boolean hasNew = false;
		
		try{
			Vector<String> xmlfiles = getXMLNames(); 
			if(xmlfiles == null) return false;
		
			for(String fname:xmlfiles){
				if(!history.contains(fname)){
					downloadFile(fname, remoteSource);
					history.addEntry(fname);
				
					hasNew = true;
				}
			}
		}
		catch(IOException e){ 
			return false;
		}
		
		return hasNew;
	}

	public boolean fetchNewData() {	
		Vector<String> xmlfiles = getLocalXMLNames();
		boolean hasNew = false;
		
		for(String fname:xmlfiles){	
			if(!history.contains(fname)){
				try {
					downloadFile(fname, localSource);
				} catch (IOException e) {
					return false;
				}
				history.addEntry(fname);
				
				hasNew = true;
			}
		}
		
		return hasNew;
	}

	private void downloadFile(String xmlFile, String source) throws IOException {
		
		InputStream fis = null;
		
		if(source.startsWith("http")){
			URL url = new URL(source + xmlFile);
			
			URLConnection conn = url.openConnection();
			conn.setConnectTimeout(GlobalParameters.TIME_OUT);
			conn.setReadTimeout(GlobalParameters.TIME_OUT);
			fis = conn.getInputStream();
		}
		else {
			fis = new FileInputStream(source + xmlFile);
		}
		
		FileOutputStream fout = new FileOutputStream(GlobalParameters.DATA_FOLDER  + xmlFile);
		  
		byte[] buf=new byte[1024];
		     
		while(true){
			int len=fis.read(buf, 0, buf.length);
			if(len<=0) break;
			fout.write(buf, 0, len); 
		}

		fis.close();

		fout.flush();     
		fout.close();
	}

	
	private Vector<String> getLocalXMLNames() /*throws IOException */{
		File file = new File(localSource);

		String files[] = file.list();
		
		Vector<String> fnames = new Vector<String>();
		
		if(files == null) return fnames;
		for(int  i=0; i<files.length; ++i){ 
			if(files[i].endsWith(".xml")) fnames.add(files[i]);
		}
			
		return fnames;
	}	
	
	private Vector<String> getXMLNames() throws IOException {
		if(remoteSource == null) return null;
		
		StringBuffer strBuffer = new StringBuffer();
 		
		URL url = new URL(remoteSource);
		InputStreamReader reader =  new InputStreamReader(url.openStream());
		
		char[] buf=new char[1024];
  
		while(true){
			int len=reader.read(buf, 0, buf.length);
			if(len<=0) break;
				
			strBuffer.append(buf, 0, len);
		}

		reader.close();
		
		String []str = strBuffer.toString().split("xml</a>");

		Vector<String> fnames = new Vector<String>(); 
		for(int  i=0; i<str.length - 1; ++i)
			fnames.add(findFileName(str[i]) + ".xml");

		return fnames;
	}
	
	private String findFileName(String fname){
		return fname.substring(fname.indexOf(".xml\">") + 6, fname.lastIndexOf("."));
	}

	public void showSelection() {
		//fileList.getSelectedValues();
		parent.cleanStrokes();
		fileList.showSelections();
	}

	public void assignSelection() {
		String source = ((GestureBrowser)getParent()).getSource();
		
		fileList.assignSelections(source);
		history.save();
	}

	public void unassignSelection() {
		fileList.unassignSelections();
		history.save();
	}

	public boolean scoreExists() {
		return scoreExists;
	}

	public void registerTabletFile(String filename) {
		history.addEntry(filename);
		history.save();
		showHistory();
	}


	
}
