package inria.paperoles.ui;

import inria.paperoles.scoremodel.GraphVoiceLine;
import inria.paperoles.scoremodel.Measure;
import inria.paperoles.scoremodel.MultiplePageSkeleton;
import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.scoremodel.ScoreSkeleton;
import inria.paperoles.scoremodel.Segment;
import inria.paperoles.scoremodel.VoiceLine;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Image;

import edu.umd.cs.piccolo.nodes.PImage;
import edu.umd.cs.piccolo.nodes.PPath;

public class GraphPaper extends SheetImage{
	
	// for simplicity, I have just some default values
	static float width = 845f, height = 620f;
	public static float off_x = 64f, off_y = 65f;
	
	Color color = new Color(110, 210, 180);
	public final static float CENTIMETER_STEP = 28.1f;
	
	static int heightInCentimeters = 17, widthInCentimeters = 26;
	
	public GraphPaper(){
		
		super();

		setStrokePaint(color);
		
		drawMainGrid();	
		drawSecondaryGrid(color);
	}
	
	@Override
	public double getHeight() {
		return height;
	}

	static public int getHeightInCent() {
		return heightInCentimeters;
	}
	
	@Override
	public double getWidth() {
		// TODO Auto-generated method stub
		return width;
	}
	
	static public ScoreSkeleton createSkeleton(MultiplePageSkeleton multiskeleton){
		
		ScoreSkeleton skeleton = new ScoreSkeleton(multiskeleton, true);
		PolyStaff polyStaff = new PolyStaff(skeleton, 1, (int)off_x, (int)off_y, 
				(int)Math.round(CENTIMETER_STEP*widthInCentimeters), 
				(int)Math.round(CENTIMETER_STEP*heightInCentimeters));
		skeleton.addDocPart(polyStaff);
		
		VoiceLine vline= new GraphVoiceLine(polyStaff, 1, (int)off_x, (int)off_y, 
				(int)Math.round(CENTIMETER_STEP*widthInCentimeters), 
				(int)Math.round(CENTIMETER_STEP*heightInCentimeters));
		skeleton.addDocPart(vline);
		
		
		Measure measure = new Measure();
		Segment segment = new Segment();
		measure.initialize(polyStaff, (int)off_x,
				(int)Math.round(CENTIMETER_STEP*heightInCentimeters));
		skeleton.addDocPart(measure);

		segment.initialize(measure, vline, (int)off_x, (int)off_y, 
				(int)Math.round(CENTIMETER_STEP*widthInCentimeters), 
				(int)Math.round(CENTIMETER_STEP*heightInCentimeters));
		skeleton.addDocPart(segment);		
		
		multiskeleton.addPageSkeleton(skeleton);
		
		return skeleton;
	}
	
	private void drawMainGrid(){
		setStroke(new BasicStroke((float) 1.4));
		// (1) create all the horizontal lines for centimeters
		for(int i = 0; i<=heightInCentimeters; ++i){
			
			float y = (float) (off_y + CENTIMETER_STEP*i);
				
			moveTo(off_x, y);
			lineTo(off_x + CENTIMETER_STEP*widthInCentimeters, y);
		}
		
		// (2) create all the vertical lines for centimeters
		for(int i = 0; i<=widthInCentimeters; ++i){
			
			float x = (float) (off_x + CENTIMETER_STEP*i);
				
			moveTo(x, off_y);
			lineTo(x, off_y + CENTIMETER_STEP*heightInCentimeters);
		}
		
	}
	
	private void drawSecondaryGrid(Color color){
		PPath grid = new PPath();
		
		grid.setStrokePaint(color);
		grid.setStroke(new BasicStroke(.7f));
		
		// (3) create mid horizontal lines for centimeters
		for(int i = 0; i<heightInCentimeters; ++i){
			
			float y = (float) (off_y + CENTIMETER_STEP*(.5 + i));
				
			grid.moveTo(off_x, y);
			grid.lineTo(off_x + CENTIMETER_STEP*widthInCentimeters, y);
		}
		
		// (4) create mid vertical lines for centimeters
		for(int i = 0; i<widthInCentimeters; ++i){
			
			float x = (float) (off_x + CENTIMETER_STEP*(.5 + i));
				
			grid.moveTo(x, off_y);
			grid.lineTo(x, off_y + CENTIMETER_STEP*heightInCentimeters);
		}
		
		addChild(grid);
	}
}
