package inria.paperoles.ui;

import inria.paperoles.customization.CommandArgumentSet;
import inria.paperoles.customization.GestureManipulationCommand;
import inria.paperoles.customization.ScopeRefinementCommand;
import inria.paperoles.ui.menus.JMarkingMenu;
import inria.paperoles.ui.menus.JMarkingMenuOption;

import java.awt.Color;
import java.awt.Font;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Vector;

import edu.umd.cs.piccolo.PCamera;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.nodes.PPath;

public class MarkingMenu extends PPath{

	GesturePath gpath;
	final static int RADIUS = 60, RADIUS2 = 120;
	
	Point2D center;
	double scale;
	JMarkingMenu menu;
	InteractionSurface interactionSurface;

	Point2D offset;
	
	public MarkingMenu(InteractionSurface interactionSurface, Point2D absoluteCenter, Point2D center, GesturePath gpath, Vector<CommandArgumentSet>options,
			double scale){
			
		this.scale = scale;
		this.gpath = gpath;
		this.center = center;

		offset = absoluteCenter;
		this.interactionSurface = interactionSurface;
		
		/// change that!!!!
		menu = new JMarkingMenu(absoluteCenter, 0);
		
		for(int i=0; i<options.size(); ++i){
			double angle = Math.PI*(.5 - ((double)2*i)/options.size());
			double cos = Math.cos(angle);
			double sin = Math.sin(angle);
			
			double dx = RADIUS *cos;
			double dy = - RADIUS *sin;
			
			CommandArgumentSet command = options.elementAt(i);
			
			JMarkingMenuOption visualLabel = new JMarkingMenuOption(command.getCommand().getName(), dx, dy);
			menu.addOption(visualLabel);

			
			FlatSubmenu submenu = null;
			if(command.getArgument() != null){
				submenu = new FlatSubmenu(interactionSurface, new Point2D.Double(absoluteCenter.getX()-cos*RADIUS2, absoluteCenter.getY()+sin*RADIUS2),
						new Point2D.Double(center.getX()-cos*RADIUS2, center.getY()+sin*RADIUS2),
						2*RADIUS2,
						angle,  Math.PI/12, gpath, command, scale);

			}
			
			MarkingMenuOption option;
			if(command.getCommand() instanceof ScopeRefinementCommand){
				option = new ScopeRefinementOption(visualLabel, command.getCommand(), submenu,
						center.getX() + dx, center.getY() + dy);
			}
			else 
				option = new MarkingMenuOption(visualLabel, command.getCommand(), submenu,
					center.getX() + dx, center.getY() + dy);
			
			addChild(option);
			//option.setVisualComponent(visualLabel);	
		
			if(submenu!=null && !submenu.hasOptions() || !command.getCommand().isActive())
				option.setInactive();
		}
		
		interactionSurface.addMenu(menu);
		scaleAboutPoint(1/scale, center);
		
		//getRoot().get
	}


	public void delete(){
		removeFromParent();
		menu.remove();
		interactionSurface.removeActiveMenu();
	}
	
	public void handleEvent(PInputEvent event){

		if(hidden){
			activeOption.handleEvent(event);
			return;
		}
			
		MarkingMenuOption option = findClosestOption(event.getPosition());

		if(activeOption == option || secondaryOption == option && option!=null) return;
		
		if(activeOption != null && enabled) activeOption.loseFocus();
		else if(secondaryOption!=null) secondaryOption.loseFocus();


		if(enabled && option != null){
			activeOption = option;
			option.gainFocus();
		}
		else if(enabled){
			activeOption = option;
		}
		else if(option != null){
			secondaryOption = option;
			option.gainFocus();
		}
		else{
			activeOption.loseFocus();
			activeOption = null;
		}
	}
	
	
	private MarkingMenuOption activeOption = null;
	private MarkingMenuOption secondaryOption = null;
	
	private MarkingMenuOption findClosestOption(Point2D p){
		if(center.distance(p) < RADIUS/(2*scale)) return null;
		
		if(!enabled) return activeOption.findClosestOption(p);
		
		double mindist2 = Double.MAX_VALUE;
		MarkingMenuOption closestOption = null;
		
		for(Object child:getChildrenReference()){
			MarkingMenuOption option = (MarkingMenuOption)child;
			
			double dist2 = Math.pow(option.getPosition().getX() - p.getX(), 2) + Math.pow(option.getPosition().getY() - p.getY(), 2);
			
			if(dist2 < mindist2){
				mindist2 = dist2;
				closestOption = option;
			}
		}
		
		if(closestOption.active) return closestOption;
		else return null;
	}

	public  GestureManipulationCommand getSelectedOption(){
		if(activeOption == null) return null;
		return activeOption.getOption();
	}
	
	
	public Object getOptionArgument(){
		if(secondaryOption!=null) return secondaryOption.getArgument();
		else return null;
	}

	boolean enabled = true;
	public void disable(){
		menu.enableMenu(false);
		enabled = false;
	}
	
	boolean hidden = false;
	public void hide(){
		hidden = true;
		menu.hide();
	}
	
	public void enable(){
		menu.enableMenu(true);
		enabled = true;
		secondaryOption = null;
	}
}
