package inria.paperoles.ui;

import inria.paperoles.gestures.StrokeAnalyzer;
import inria.paperoles.grammar.SyntaxAnalyzer;
import inria.paperoles.grammar.Token;
import inria.paperoles.operations.MatchedOperation;
import inria.paperoles.scoremodel.MultiplePageSkeleton;

import java.awt.Component;
import java.awt.Dimension;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Vector;

public class MultiplePageViewer {

	Vector<ScoreViewer> scoreViewers;
//	Hashtable<Integer, ScoreViewer> graphViewers;
	GestureBrowser gbrowser;
	MultiplePageSkeleton mskeleton;
	StrokeAnalyzer strokeAnalyzer;
	SyntaxAnalyzer syntaxAnalyzer;
	
	public MultiplePageViewer(GestureBrowser gbrowser){
		scoreViewers = new Vector<ScoreViewer>();
//		graphViewers = new Hashtable<Integer, ScoreViewer>();
		
		this.gbrowser = gbrowser;
	}
	
	public GestureBrowser getGestureBrowser(){
		return gbrowser;
	}
	
	public void setSkeleton(MultiplePageSkeleton mskeleton){
		this.mskeleton = mskeleton;
	}
	
	public void setAnalyzers(StrokeAnalyzer strokeAnalyzer, SyntaxAnalyzer syntaxAnalyzer){
		this.strokeAnalyzer = strokeAnalyzer;
		this.syntaxAnalyzer = syntaxAnalyzer;
	}
	
	public StrokeAnalyzer getStrokeAnalyzer(){
		return strokeAnalyzer;	
	}
	
	public SyntaxAnalyzer getSyntaxAnalyzer(){
		return syntaxAnalyzer;
	}
	
	public MultiplePageSkeleton getSkeleton(){
		return mskeleton;
	}
	
	public int getScoreViewersNum(){
		return scoreViewers.size();
	}
	
/*	public int getGraphViewersNum(){
		return graphViewers.size();
	}	
*/	
	public void addPageViewer(ScoreViewer pageViewer){
		scoreViewers.add(pageViewer);
	}
	
/*	
	public ScoreViewer getGraphViewer(int pnum){
		return graphViewers.get(pnum);
	}
	
	public void addGraphViewer(ScoreViewer graphViewer, int num){
		graphViewers.put(num, graphViewer);
	}
*/	
	public Vector<ScoreViewer> getPageViewers(){
		return scoreViewers;
	}

	/*
	public Collection<ScoreViewer> getGraphViewers(){
		return graphViewers.values();
	}*/

	public void zoom(boolean b) {
		for(ScoreViewer pageViewer:scoreViewers)
			pageViewer.zoom(b);
		
/*		for(ScoreViewer pageViewer:graphViewers.values())
			pageViewer.zoom(b);
*/		
		if(!scoreViewers.isEmpty()){
			scoreViewers.elementAt(0).getParent().setPreferredSize(getSize());
			scoreViewers.elementAt(0).getRootPane().updateUI();
		}
	}

	public void activatePenLayer(boolean b) {
		//if(!b) analyzer.recognizeSyntax();
		//else analyzer.renew();
		
		if(!b) refresh();
		
		for(ScoreViewer pageViewer:scoreViewers)
			pageViewer.activatePenLayer(b);
		
//		for(ScoreViewer pageViewer:graphViewers.values())
//			pageViewer.activatePenLayer(b);
	}

	public void activateMouseCoords(boolean b) {
		for(ScoreViewer pageViewer:scoreViewers)
			pageViewer.activateMouseCoords(b);
		
//		for(ScoreViewer pageViewer:graphViewers.values())
//			pageViewer.activateMouseCoords(b);
	}

	public void showSkeleton(boolean b) {
		for(ScoreViewer pageViewer:scoreViewers)
			pageViewer.showSkeleton(b);
		
//		for(ScoreViewer pageViewer:graphViewers.values())
//			pageViewer.showSkeleton(b);
	}
	
	public void showScopes(boolean b) {
		for(ScoreViewer pageViewer:scoreViewers)
			pageViewer.showScopes(b);
		
//		for(ScoreViewer pageViewer:graphViewers.values())
//			pageViewer.showScopes(b);
	}
	
	public ScoreViewer get(int i){
		return scoreViewers.elementAt(i);
	}

	public int getPageNum(){
		return scoreViewers.size();
	}
	
	public Dimension getSize() {
		if(scoreViewers.isEmpty()) return new Dimension(0, 0);
		// TODO Auto-generated method stub
		Dimension dim = scoreViewers.elementAt(0).getMaximumSize();
		return new Dimension((int)dim.getWidth(),
				(int)(dim.getHeight()*scoreViewers.size()));
	}

	SyntaxAnalyzer analyzer;
	public void setSyntaxAnalyzer(SyntaxAnalyzer syntaxAnalyzer) {
		this.analyzer = syntaxAnalyzer;
		
	}

	public void refresh() {
		analyzer.recognizeSyntax();
		for(ScoreViewer pageViewer:scoreViewers)
			pageViewer.refresh();

//		for(ScoreViewer pageViewer:graphViewers.values())
//			pageViewer.refresh();
		
		analyzer.renew();
	}

	public void clear() {
		scoreViewers.removeAllElements();
		
//		graphViewers.clear();
	}
	
	public void cleanWrittenStrokes(){
		for(ScoreViewer pageViewer:scoreViewers)
			pageViewer.clearGestures();
		
//		for(ScoreViewer pageViewer:graphViewers.values())
//			pageViewer.clearGestures();
		
		analyzer.clean();
		
		mskeleton.clean();
	}

	public Collection<Token> getMatchedTokens() {
		Vector<Token> matched = new Vector<Token>();
		
		for(ScoreViewer pageViewer:scoreViewers){
			matched.addAll(pageViewer.gesturesCanvas.getMatchedTokens());
		}
		
		return matched;
	}

	public void resetDefinitions() {
		for(ScoreViewer pageViewer:scoreViewers)
			pageViewer.gesturesCanvas.cleanRecognition();

//		for(ScoreViewer pageViewer:graphViewers.values())
//			pageViewer.gesturesCanvas.cleanRecognition();	
	}

}
