package inria.paperoles.ui;

import inria.paperoles.operations.Operation;
import inria.paperoles.operations.OperationArgument;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.*;

public class OperationDefinitionDialog extends JDialog{
	
	final static String POINTS = "points: ((x1...xn) (y1...yn))";
	final static String X_RANGE = "x-range: (x1 xn)";
	final static String START = "start point: (x1 y1)";
	final static String CENTER = "central point: (xc yc)";
	final static String PARAMETER = "list of circled parameters: str1...strn";
	
	/*
	final static String GESTURE_ID = "gesture-id: id";
	final static String GESTURE = "gesture: (id (x1...xn) (y1...yn))";
	*/
	
	//operationName;
	Operation operation;
	final static Font textFont = new Font("Arial", Font.PLAIN, 11);
	OperationsList operationsList;
	
	public OperationDefinitionDialog(GestureBrowser parentFrame, int x, int y){
		this(parentFrame, x, y, null);
	}
	
	public OperationDefinitionDialog(GestureBrowser parentFrame, int x, int y, Operation op){
		super(parentFrame, "Add Operation", true);
		operationsList = parentFrame.functionsBar.list;
		
		if(op == null) operation = new Operation();
		else operation = op;
		
		getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
				
	    setUndecorated(true);
	    setResizable(false);
	    
	    JLabel label = new JLabel("Identifier ");
	    JTextField  operationName = new JTextField(8);
	    operationName.setText(operation.getName());
	    operationName.setFont(textFont);
	    
		operationName.addKeyListener(new KeyAdapter(){
			public void keyReleased(KeyEvent e) { 
				
				String text = ((JTextField)e.getSource()).getText();
		
				if(operationsList.contains(text.trim())){
					addButton.setEnabled(false);
				}
				else if(text.trim().equals("")) 
					addButton.setEnabled(false);
				else{
					operation.setName(text.trim());
					addButton.setEnabled(true);
				}
			}
		});
	    
		operationName.setMaximumSize(operationName.getMinimumSize());
		JPanel namePanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		namePanel.add(label);
		namePanel.add(operationName);
//		operationName.setBackground(new Color(210, 210, 210));
		namePanel.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
		
		//////////
/*
		namePanel.add(new JLabel(" Return"));
		String comboBoxItems[] = {SYMBOLS, POINTER, TEXT, WAVE, WAVEPOINTS, NONE};
        JComboBox returnType = new JComboBox(comboBoxItems);
        returnType.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JComboBox cb = (JComboBox)e.getSource();
                operation.setReturnType(cb.getSelectedIndex());
            }
        });
        returnType.setFont(textFont);
        returnType.setEditable(false);
        returnType.setSelectedIndex(5);
        namePanel.add(returnType);
 */       
		//////////
		getContentPane().add(namePanel);
	    ParameterDefinitionPanel defPanel = new ParameterDefinitionPanel(this, operation);
	    JPanel bottomPanel = createBottomPanel(op == null);
	    getContentPane().add(defPanel);
	    getContentPane().add(bottomPanel);
	   
	    namePanel.setAlignmentX(LEFT_ALIGNMENT);
	    defPanel.setAlignmentX(LEFT_ALIGNMENT);
	    bottomPanel.setAlignmentX(LEFT_ALIGNMENT);
	    
		setLocation(x - (int)getPreferredSize().getWidth() + parentFrame.getX(), y);
		
	    pack();
	}
	
	
	JButton addButton;
	private JPanel createBottomPanel(final boolean toAdd) {
		
		JPanel panel = new JPanel();
		
		addButton = new JButton("Confirm");
		addButton.setFont(new Font("Arial", Font.PLAIN, 12));
		if(toAdd) addButton.setEnabled(false);

		JButton closeButton = new JButton("Cancel");
		closeButton.setFont(new Font("Arial", Font.PLAIN, 12));
		
		addButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(toAdd) operationsList.addOperation(operation);
				closeDialog();
			}
		});
		
		closeButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(!toAdd) operationsList.cancelEdit();
				closeDialog();
			}
		});
		
		panel.add(addButton);
		panel.add(closeButton);
		
		return panel;
	}
	
	private void closeDialog(){
		super.dispose();
	}	
	
	
	class InsertParameterButton extends JButton{
		ParameterDefinitionPanel parent;
		
		InsertParameterButton(final ParameterDefinitionPanel parent){
			super("+");
			
			this.parent = parent;
			setToolTipText("Insert new parameter");
			setBorderPainted(false);
			
			addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					OperationArgument argument = new OperationArgument(parent.operation);
					
					int index = getIndex(), pos;
					parent.operation.addArgument(argument, pos=(index - 4)/8);
					parent.addArgument(argument, index + 4, pos);
					
					parent.refresh();
				}
			});
		}
		
		private int getIndex(){			
			for(int i = 0; i<parent.getComponentCount(); ++i)
				if(parent.getComponent(i) == this) return i;
			
			return 0;
		}
		
	}

	class RemoveParameterButton extends JButton{
		ParameterDefinitionPanel parent;
		
		RemoveParameterButton(final ParameterDefinitionPanel parent, final OperationArgument arg){
			super("-");
			
			this.parent = parent;
			setToolTipText("Remove parameter");
			setBorderPainted(false);
			
			addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					parent.removeArgument(arg, getIndexOfComponent());
					
					parent.refresh();
				}
			});
		}
		
		private int getIndexOfComponent(){			
			for(int i = 0; i<parent.getComponentCount(); ++i)
				if(parent.getComponent(i) == this) return i;
			
			return 0;
		}
	}
	

	class ParameterDefinitionPanel extends JPanel{
		JDialog dialog;
		Operation operation;
		
		ParameterDefinitionPanel(JDialog dialog, Operation operation){		
			this.operation = operation;
			this.dialog = dialog;
			
			setBorder(BorderFactory.createTitledBorder("Parameters"));
			setLayout(new SpringLayout());
			
			numbers = new Vector<JLabel>();
			
			//add header
			add(new JLabel(""));
//			add(new JLabel("marker (optional)      "));
			add(new JLabel("                 "));
			add(new JLabel("type                                              "));
			add(new JLabel(""));
			
			addPlusButton(null, getComponentCount());
			addArguments();
			/*
			SpringUtilities.makeCompactGrid(this, //parent
	                2, 4,
	                3, 3,  //initX, initY
	                3, 3); //xPad, yPad*/
		}	
		
		public void refresh() {
			SpringUtilities.makeCompactGrid(this, //parent
	                2*operation.getArguments().size() + 2, 4,
	                3, 3,  //initX, initY
	                3, 3); //xPad, yPad
			
			validate();
			dialog.pack();
		}

		private void addPlusButton(JComponent componentBefore, int index){
			add(new InsertParameterButton(this), index);
			add(new JLabel(""), index + 1);
			add(new JLabel(""), index +  2);
			add(new JLabel(""), index + 3);
		}
		
		
		void removeArgument(OperationArgument argument, int index){
			int num = operation.getArguments().indexOf(argument);
			operation.removeArgument(argument);
			numbers.remove(num);
			updateLabelNums();
			
			for(int i=0; i<8; ++i) remove(index);
		}
		
		Vector<JLabel> numbers;
		private void updateLabelNums(){
			for(int i = 0; i<numbers.size(); ++i)
				numbers.elementAt(i).setText((i+1) +":");
		}
		
		
		void addArgument(final OperationArgument argument, int index, int num){
			
			add(new RemoveParameterButton(this, argument), index);
			
			JLabel param = new JLabel("");
			JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
			
			JLabel lnum = new JLabel();
			numbers.insertElementAt(lnum, num);
			updateLabelNums();
			panel.add(lnum);
			panel.add(param);
		    
		    add(panel, index + 1);
		    
	        String comboBoxItems[] = {POINTS, X_RANGE, START, CENTER, PARAMETER };
	        JComboBox paramType = new JComboBox(comboBoxItems);
	        paramType.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent e) {
	                JComboBox cb = (JComboBox)e.getSource();
	                argument.setType(cb.getSelectedIndex());
	            }
	        });
	        paramType.setFont(textFont);
	        paramType.setEditable(false);
	        paramType.setSelectedIndex(argument.getType());
	        panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
	        panel.add(paramType);
	        add(panel, index + 2);
	        
	        panel = new JPanel();
	        JCheckBox box = new JCheckBox();
	        box.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent e) {
	            	JCheckBox cb = (JCheckBox)e.getSource();
	            	argument.setRepeated(cb.isSelected());
	            }
	        });
	        panel.add(box);
	        box.setEnabled(false);
	        
	        add(panel, index + 3);
	        
	        if(argument.isRepeated()) box.setSelected(true);
	        
	        addPlusButton(box, index + 4);
		}
		
		/*
		void addArgument(final OperationArgument argument, int index, int num){
			
			add(new RemoveParameterButton(this, argument), index);
			
			JTextField param = new JTextField(7);
			JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
			
			JLabel lnum = new JLabel();
			numbers.insertElementAt(lnum, num);
			updateLabelNums();
			panel.add(lnum);
			panel.add(param);
		    param.setFont(textFont);
	//	    param.setBackground(new Color(210, 210, 210));
		    param.setText(argument.getParameterName());
		    
			param.addKeyListener(new KeyAdapter(){
				public void keyReleased(KeyEvent e) { 
					String text = ((JTextField)e.getSource()).getText();
			
					argument.setParameterName(text.trim());
				}
			});
		    
		    add(panel, index + 1);
		    
	        String comboBoxItems[] = {SYMBOLS, POINTER, TEXT};
	        JComboBox paramType = new JComboBox(comboBoxItems);
	        paramType.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent e) {
	                JComboBox cb = (JComboBox)e.getSource();
	                argument.setType(cb.getSelectedIndex());
	            }
	        });
	        paramType.setFont(textFont);
	        paramType.setEditable(false);
	        paramType.setSelectedIndex(argument.getType());
	        panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
	        panel.add(paramType);
	        add(panel, index + 2);
	        
	        panel = new JPanel();
	        JCheckBox box = new JCheckBox();
	        box.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent e) {
	            	JCheckBox cb = (JCheckBox)e.getSource();
	            	argument.setRepeated(cb.isSelected());
	            }
	        });
	        panel.add(box);
	        add(panel, index + 3);
	        
	        if(argument.isRepeated()) box.setSelected(true);
	        
	        addPlusButton(box, index + 4);
		}
		*/
		
		void addArguments(){
			int i = 0;
			for(OperationArgument argument:operation.getArguments())
				addArgument(argument, 8*(i+1), i++);

			SpringUtilities.makeCompactGrid(this, //parent
			                                2 + 2*operation.getArguments().size(), 4,
			                                3, 3,  //initX, initY
			                                3, 3); //xPad, yPad
		}
			
	}
}


