package inria.paperoles.ui;

import inria.paperoles.customization.GestureManipulationCommand;
import inria.paperoles.customization.GestureRecognitionCommand;
import inria.paperoles.customization.ParameterRecognitionCommand;
import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.SingleInkStroke;
import inria.paperoles.gestures.recognition.GeneralizedGestureClass;
import inria.paperoles.grammar.ComplexGesture;
import inria.paperoles.grammar.GroupingGesture;
import inria.paperoles.grammar.Token;
import inria.paperoles.scoremodel.GestureNote;
import inria.paperoles.scoremodel.ScoreSkeleton;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JFrame;

import org.xml.sax.SAXException;

import papertoolkit.paper.Region;
import papertoolkit.paper.Sheet;
import papertoolkit.pen.ink.InkStroke;

import edu.umd.cs.piccolo.PCanvas;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.event.PInputEventListener;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.nodes.PText;

public class ScoreViewer extends PCanvas implements PInputEventListener{

	private final static Color OVERLAY_INK_COLOR = Color.blue;
	private final static Color TABLET_COLOR = new Color(120, 30, 120);
	private final static Color OVERLAY_BACK_COLOR = new Color(160, 160, 220, 50);
	private final static double SCALE_STEP = 1.2;
	
	Vector<InkStroke> strokesToSave;
	
	SheetImage sheetIm;
	GesturesCanvas gesturesCanvas;
	PText mouseIndex;
	ScoreSkeleton scoreSkeleton;
	DrawingCanvas drawingCanvas;

	int pageNum;
	
	GraphsBar graphsBar;
	
	PaperTemplate ptemplate;
	
	JFrame parentFrame;
	
	Hashtable<ScoreViewer, GraphButton> graphButtons;
	
	
	public ScoreViewer(JFrame frame, int pageNum, Image sheetImage, ScoreSkeleton scoreSkeleton, boolean isVertical,
			PaperTemplate ptemplate){
		super();
	
		parentFrame = frame;
		this.ptemplate = ptemplate;
		this.pageNum = pageNum;
		this.scoreSkeleton = scoreSkeleton;

		if(GlobalParameters.isTest) sheetIm = new PDFImage("sheets/photo.jpg");
		else if(sheetImage!=null) sheetIm = new PDFImage(sheetImage);
		
		// this is the case of graph paper (papier millimetre)
		else {
			sheetIm = new GraphPaper();
			
			graphButtons = new Hashtable<ScoreViewer, GraphButton>();
		}
			
		gesturesCanvas = new GesturesCanvas(scoreSkeleton);
		drawingCanvas = new DrawingCanvas();
		drawingCanvas.setStrokePaint(TABLET_COLOR);
		
		mouseIndex = new PText();	
		
		getLayer().addChild(sheetIm);
		getLayer().addChild(scoreSkeleton);
		getLayer().addChild(drawingCanvas);
		getLayer().addChild(gesturesCanvas);
		
		graphsBar = new GraphsBar(this, sheetIm.getWidth(), sheetIm.getHeight());
		getLayer().addChild(graphsBar);
		
		getLayer().addChild(mouseIndex);
		
//		gesturesCanvas.setNotationLayer(drawingCanvas);
		
		scoreSkeleton.setVisible(false);
		mouseIndex.setVisible(false);
		mouseIndex.setPaint(java.awt.Color.white);
		
		initiateScale(/*.63f*/1f);
		
		removeInputEventListener(getZoomEventHandler());
		removeInputEventListener(getPanEventHandler());	
		addInputEventListener(this);
		
		setMaximumSize(getSheetSize());
		setPreferredSize(getSheetSize());

		
		strokesToSave = new Vector<InkStroke>();
		this.isVertical = isVertical;
	}
	
	public void addButton(ScoreViewer viewer, GraphButton button){
		graphButtons.put(viewer, button);
	}
	
	public void highlightButton(ScoreViewer viewer, boolean b){
		if(graphButtons != null && this != viewer && viewer != null){
			GraphButton button = graphButtons.get(viewer);
			if(button != null) button.highlight(b);
		}
	}
	
	public void addGraphViewer(ScoreViewer graphViewer){
		graphsBar.addGraph(graphViewer);
	}
	
	public void addGraphViewerSeparator(){
		graphsBar.addSeparator();
	}
	
	public boolean isGraphViewer(){
		return sheetIm instanceof GraphPaper; 
	}
	
	public GestureBrowser getTopLevelAncestor(){
		if(parentFrame instanceof GestureBrowser) return (GestureBrowser)parentFrame;
		else return (GestureBrowser)parentFrame.getParent();
	}
	
	public void initDraw(Point2D p){
		drawingCanvas.moveTo((float)p.getX(), (float)p.getY());
	}
	
	public void draw(Point2D p){
		drawingCanvas.lineTo((float)p.getX(), (float)p.getY());
	}
	
	public JFrame getParentFrame(){
		return parentFrame;
	}
	
	public Dimension getSheetSize(){
		return new Dimension((int)sheetIm.getWidth(), (int)sheetIm.getHeight());
	}
	
	/*public void addNotationSymbol(GestureNote gnote){
		notationCanvas.addChild(gnote);
	}*/
	
	void initiateScale(float scale){
		getLayer().getCamera(0).setViewScale(scale);
	}

	public double getScale(){
		return getLayer().getCamera(0).getViewScale();
	}

	public void zoom(boolean in) {
		double scale; 
		
		if(in) scale = getLayer().getCamera(0).getViewScale()*SCALE_STEP;
		else scale = getLayer().getCamera(0).getViewScale()/SCALE_STEP;
		
		getLayer().getCamera(0).setViewScale(scale);
		
		if(sheet!=null)
			for (Region r : sheet.getRegions()) {	
				r.setScale(scale, scale);
				//	DrawingSurface.setGlobalScrolling(scrollPosition.getX(), scrollPosition.getY());
			}
		
		setBounds(getX(), getY(), (int)Math.round(scale*getSheetSize().getWidth()), 
				(int)Math.round(scale*getSheetSize().getHeight()));
		
		setMaximumSize(new Dimension(getWidth(), getHeight()));
	//	getParent().setPreferredSize(new Dimension(getWidth(), getHeight()));
	//	getRootPane().updateUI();
	}
	
	
	boolean isPenEnabled = false;
	public void recordStroke(InkStroke stroke, boolean penEnabled){
		strokesToSave.add(stroke);
		
		isPenEnabled = penEnabled;
	}
	
	public boolean isPenEnabled(){
		return isPenEnabled;
	}
	
	public void acceptStroke(SingleInkStroke stroke, boolean realPenData){
		gesturesCanvas.addNewStroke(stroke, realPenData);	
		
		if(surface != null) surface.cleanAll();
	}

	public Vector<InkStroke> getAcceptedStrokes(){
		return strokesToSave;
	}
	
	Sheet sheet;
	DrawingSurface surface;
	
	public boolean hasActiveSurface(){
		return surface == DrawingSurface.getActiveSurface();
	}
	
	public void enableWriting(Sheet sheet, int width, int height){

		this.sheet = sheet;
		
		// There should be a single only region!!!
		for (Region r : sheet.getRegions()) {		
			surface = new DrawingSurface(r);
			surface.setBackground(OVERLAY_BACK_COLOR);
			surface.setForeground(OVERLAY_INK_COLOR);

			add(surface);
			surface.setBounds(0, 0, width, height);
			
			surface.setVisible(false);
		}
	}

	public GesturesCanvas getCanvas(){
		return gesturesCanvas;
	}
	
	public void updateView(Point viewPosition) {
		DrawingSurface.setGlobalScrolling(viewPosition.getX(), viewPosition.getY());
		gesturesCanvas.setViewPosition(viewPosition);
	}

	public void activatePenLayer(boolean b) {		
	
	//	surface.setBounds(0, 0, sheet.getWidth(), getHeight());
		
		gesturesCanvas.setInWriting(b);
		surface.setVisible(b);
	}
	
	public void refresh(){
		gesturesCanvas.refresh();
	}
	
	// output to store the score coordinates
	BufferedWriter output;
	public final static String DIRPATH = "descr/"; 
	public final static String FILENAME = "scorexxx";
	Point2D p_temp, p1 = null;
	private boolean isVertical = true;
	
	public void activateMouseCoords(boolean b)  {
		mouseIndex.setVisible(b);

		try{
			if(!b){
				output.close();
				return;
			}
		
			output = new BufferedWriter(new FileWriter(DIRPATH + FILENAME, true));
		}
		catch(IOException e){e.printStackTrace();}
		
		KeyListener listener = new KeyListener(){
			public void keyPressed(KeyEvent e) {
				try{
					switch(e.getKeyCode()){
						case KeyEvent.VK_S:
							mouseIndex.getX();
							output.write("(chord " + mouseIndex.getText() + " 8 8)\n");
							output.write("(note " + mouseIndex.getText() + " 8 8)\n");
							break;
							
						case KeyEvent.VK_0:
							p1 = p_temp;
							break;
							
						case KeyEvent.VK_1:
							if(p1 == null) output.write("...error");
							
							output.write("(staff " + (int)p1.getX() + " " + (int)p1.getY() + " " + (int)(p_temp.getX() - p1.getX()) + " " +
									 (int)(p_temp.getY() - p1.getY()) + ")\n");
							break;	
							
						case KeyEvent.VK_2:
							if(p1 == null) output.write("...error");
							
							output.write("(measure " + (int)p1.getX() + " " + (int)p1.getY() + " " + (int)(p_temp.getX() - p1.getX()) + " " +
									 (int)(p_temp.getY() - p1.getY()) + ")\n");
							break;								
							
						case KeyEvent.VK_L:
							output.write("\n");
							break;
					}
				}
				catch(IOException e1){e1.printStackTrace();}
			}

			public void keyReleased(KeyEvent e) {
				// TODO Auto-generated method stub
				
			}

			public void keyTyped(KeyEvent e) {
				// TODO Auto-generated method stub
				
			}
			
		};

		for(KeyListener kl:getKeyListeners()) removeKeyListener(kl);
		
		addKeyListener(listener);
	}

	public void clearGestures(){
		gesturesCanvas.cleanSpace();
		strokesToSave.clear();
//		notationCanvas.removeAllChildren();
	}
	
	public PaperTemplate getPaperTemplate(){
		return ptemplate;
	}

	public void processEvent(PInputEvent e, int arg) {
		if(!mouseIndex.getVisible()) return;

		p_temp = e.getPosition();
		mouseIndex.setText((int)e.getPosition().getX() + " "+ (int)e.getPosition().getY());
		mouseIndex.setX(e.getPosition().getX());
		mouseIndex.setY(e.getPosition().getY() - 20);
	}

	public void showSkeleton(boolean b) {
		scoreSkeleton.setVisible(b);
		sheetIm.setVisible(!b);
//		if(notationCanvas!=null) notationCanvas.setVisible(!b);
	}

	public void showScopes(boolean b) {
		Iterator childIterator = gesturesCanvas.getChildrenIterator();
			
		while(childIterator.hasNext()){
			showScope((GesturePath)childIterator.next(), b);
//			PPath scope = ((GesturePath)childIterator.next()).scope;
//			if(scope!=null) scope.setVisible(b);
		}
	}
	
	private void showScope(GesturePath gpath, boolean b){	
//		if(gpath.samplesButton != null) gpath.samplesButton.setVisible(b);
		
		if(gpath.getToken() instanceof GroupingGesture) {
			showScope(((GroupingGesture)gpath.getToken()).getGrouper().getGesturePath(gesturesCanvas), b);
			PPath scope = gpath.scope;
			if(scope!=null) scope.setVisible(b);
		}
		else if(gpath.getToken() instanceof ComplexGesture){
			Vector<Token> subtokens = ((ComplexGesture)gpath.getToken()).getSubTokens();
			for(int i=1; i<subtokens.size(); ++ i)		
				showScope(subtokens.elementAt(i).getGesturePath(gesturesCanvas), b);	
		}
		else {
			PPath scope = gpath.scope;
			if(scope!=null) scope.setVisible(b);
		}
	}

	// for simplicity I consider that a score can have up to 2 pages:
	// Page 1 corresponds to page numbers: 1 - 5 and Page 2 corresponds to numbers: 6-0
/*	public boolean matches(int num) {
		if(num%10 <= 5)
			if(pageNum == 1) return true;
			else return false;
		else 
			if(pageNum == 2) return true;
			else return false;	
	}
	*/
	
	public boolean matches(int num){
	
		if(num == pageNum) return true; 
		else return false;

	
		//return true;
	//	if(num%2 + 1 == pageNum) return true; else return false;
		
//		if((num%10)/2 + 1 == pageNum) return true;
//		else return false;
	}

	public void removeStroke(GeneralInkStroke firstStroke) {
		for(SingleInkStroke stroke:firstStroke.getStrokes())
			if(stroke.getRef() != null) strokesToSave.remove(stroke.getRef());
	}

	public boolean isVertical() {
		return isVertical ;
	}

	public void cleanDrawingSurface() {
		drawingCanvas.reset();
	}
	
}
