package inria.paperoles.uievents;

import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.geom.Point2D;
import java.util.Vector;

import com.sun.org.apache.bcel.internal.generic.GETSTATIC;

import inria.paperoles.customization.ClassifyAsCommand;
import inria.paperoles.customization.CommandArgumentSet;
import inria.paperoles.customization.GestureDefinitionCommand;
import inria.paperoles.customization.GestureManipulationCommand;
import inria.paperoles.customization.GestureRecognitionCommand;
import inria.paperoles.customization.GestureRemovalCommand;
import inria.paperoles.customization.ScopeRefinementCommand;
import inria.paperoles.customization.SendToOutputCommand;
import inria.paperoles.customization.UnclassifyCommand;
import inria.paperoles.gestures.recognition.GeneralizedGestureClass;
import inria.paperoles.grammar.ArrowConnector;
import inria.paperoles.grammar.ConnectorToken;
import inria.paperoles.grammar.SelectorToken;
import inria.paperoles.openmusic.OpenMusicClient;
import inria.paperoles.operations.Operation;
import inria.paperoles.ui.GestureBrowser;
import inria.paperoles.ui.GestureHandle;
import inria.paperoles.ui.GesturePath;
import inria.paperoles.ui.GesturesCanvas;
import inria.paperoles.ui.InteractionSurface;
import inria.paperoles.ui.MarkingMenu;
import inria.paperoles.ui.ScoreViewer;

import inria.paperoles.ui.ScopeInteractor;
import edu.umd.cs.piccolo.PCamera;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.nodes.PPath;

public class GestureEventHandler extends PBasicInputEventHandler{

	GesturePath activePath;
	MarkingMenu menu;
	GesturesCanvas canvas;
	InteractionSurface interactionSurface;
	
	public GestureEventHandler(GesturesCanvas gesturesCanvas) {

		this.canvas = gesturesCanvas;
		
	}

	public void setInteractionSurface(InteractionSurface interactionSurface){
		this.interactionSurface = interactionSurface;

	}
	
	public void mouseEntered(PInputEvent event){
		
		PNode picked = event.getPickedNode();
		if(!(picked instanceof GestureHandle)) return; 
		
		GesturePath path =  (GesturePath)event.getPickedNode().getParent().getParent();		
		if(path.isDisabled()) return;
	
//System.err.print("Real Token : " + ((GesturePath)path).getToken());			
		//TODO....
	//	activePath = ((GesturePath)path).getToken().getRoot().getGesturePath(canvas);
		activePath = ((GesturePath)path).getToken().getContainerToken().getGesturePath(canvas);
//System.err.println("Container: " + activePath.getToken());		
		if(activePath.getToken() instanceof ConnectorToken && !(activePath.getToken() instanceof ArrowConnector)){
			activePath = null;
			return;
		}
			
		GesturePath rootPath = ((GesturePath)path).getToken().getRoot().getGesturePath(canvas);

		if(menu==null){
			rootPath.setFocus();
			if(activePath != rootPath) activePath.setFocused(false); 
		}
	}
	
	public void mouseMoved(PInputEvent event){}
	
	public void mouseDragged(PInputEvent event){ 
		if(menu!=null) menu.handleEvent(event);
	}
	
	public void mouseExited(PInputEvent event){
		PNode picked = event.getPickedNode(); 
		if(!(picked instanceof GestureHandle)) return; 		
		
		GesturePath path =  (GesturePath)event.getPickedNode().getParent().getParent();	
		//path = ((GesturePath)path).getToken().getRoot().getGesturePath(canvas);
		path = ((GesturePath)path).getToken().getContainerToken().getGesturePath(canvas);
	
		if(activePath == path && !activePath.isDisabled()){
			if(menu==null){
				GesturePath rootPath = activePath.getToken().getRoot().getGesturePath(canvas);
				rootPath.setNormal();
				//if(rootPath != activePath) activePath.
			}
			activePath = null;
		}
		
	}
	
	public void mouseClicked(PInputEvent event){

	}
		
	private void handleOperation(){
		GeneralizedGestureClass gclass = clickedPath.getGestureClass();
	
		if(gclass == null) clickedPath.revealToken();
		else clickedPath.revealOperation();
	}

	
	GesturePath clickedPath = null;
	public void mousePressed(PInputEvent event){ 


		if(event.isLeftMouseButton() && !event.isAltDown()){
			PNode picked = event.getPickedNode();
			if(!(picked instanceof GestureHandle)) return; 
	
			GesturePath path =  (GesturePath)event.getPickedNode().getParent().getParent();		
			if(path.isDisabled()) return;
			
			clickedPath = ((GesturePath)path).getToken().getContainerToken().getGesturePath(canvas);;
			handleOperation();
			
			return;
		}
		
		if(activePath == null  || activePath.isDisabled()) return;
				
		if(!event.isRightMouseButton() && !event.isAltDown()){ 
			return;			
		}
		
		// For the moment create all the menuOptions here....
		Vector<CommandArgumentSet> options = new Vector<CommandArgumentSet> ();
		CommandArgumentSet option1 = new CommandArgumentSet(new GestureDefinitionCommand(activePath), null);
		CommandArgumentSet option2 = new CommandArgumentSet(new ClassifyAsCommand(activePath), activePath.getAlternativeResults());
		CommandArgumentSet option3 = new CommandArgumentSet(new UnclassifyCommand(activePath), null);
		CommandArgumentSet option4 = new CommandArgumentSet(new GestureRemovalCommand(activePath), null);
//		CommandArgumentSet option5 = new CommandArgumentSet(new ScopeRefinementCommand(activePath), null);
		
		CommandArgumentSet option6 = new CommandArgumentSet(new SendToOutputCommand(activePath, interactionSurface), null);
	
	
		options.add(option1);
		options.add(option2);
		options.add(option3);
		
		options.add(option4);
//		options.add(option5);
		
		options.add(option6);
		
		menu = new MarkingMenu(interactionSurface, interactionSurface.getMousePosition(), event.getPosition(), activePath, options, canvas.getViewScale());
		//activePath.getParent().
		canvas.addChild(menu);	
					
	}
	
	public void mouseReleased(PInputEvent event){
		if(menu == null /*|| activePath == null || activePath.isDisabled()*/){
			if(event.isLeftMouseButton() && clickedPath != null) clickedPath.hideOperation();
		
			clickedPath = null;
			return;
		}
		
		GestureManipulationCommand option = menu.getSelectedOption();
		Object argument = menu.getOptionArgument();
	
//		canvas.removeChild(menu);
		menu.delete();
		menu = null;

		double x = event.getPosition().getX()*canvas.getViewScale();
		double y = event.getPosition().getY()*canvas.getViewScale();

		if(option!=null) option.activate(argument != null?argument:new Point2D.Double(x, y));
		
		GesturePath path =  (GesturePath)event.getPickedNode().getParent().getParent();	
		if(path != activePath){
			path.getToken().getRoot().getGesturePath(canvas).setNormal();
		}

		//if(activePath!=null) activePath.setFocused();
		if(activePath!=null){
			GesturePath rootPath = activePath.getToken().getRoot().getGesturePath(canvas);
			rootPath.setFocus();
			if(activePath != rootPath) activePath.setFocused(false); 
		}
	}
}
